from Numeric import *
from mutils import *
from scenarios2 import *
import mx.ODBC.Windows as ODBC

def get_volume(dsn, year):
	conn=ODBC.connect(dsn=dsn, user="", password="")
	cursor=conn.cursor()
	query='select kind, V1 as mol1, V2 as mol2, Vsr as srd, Vpr as prs, VIspper as spl_per, Vper as per from "S%s$"' % year
	#print query
	cursor.execute(query)
	#print cursor.description
	#print cursor.fetchall()
	rc=ModelParameter()
	data=cursor.fetchone()
	s=0
	while data:
		(kind, mol1, mol2, srd, prs, spl_per, per)=data
		spl=spl_per - per
		kind=int(kind)
		setattr(rc, "mol1_%i" % kind, mol1)
		setattr(rc, "mol2_%i" % kind, mol2)
		setattr(rc, "srd_%i" % kind, srd)
		setattr(rc, "prs_%i" % kind, prs)
		setattr(rc, "spl_%i" % kind, spl)
		setattr(rc, "per_%i" % kind, per)
		s+=mol1+ mol2+ srd+ prs+ spl_per
		data=cursor.fetchone()
	print "Summ vol:", s
	return ModelParameter({"V":rc})

START_VOL=get_volume("forest",1973)
START_VOL.save("START_VOL.py","START_VOL")

def concentration(vol, traj):
	s=scen_low_low_low_low.m_59[0]
	return float(vol.V)/float(s.S)

next_data={
	"low":(0,"middle"),
	"middle":(0,"high"),
	"high":(1,"low"),
}

def get_next_descr(prev):
	answer=[]
	p=list(prev)
	p.reverse()
	carry=1
	for d in p:
		if carry:
			(carry, next)=next_data[d]
		else:
			next=d
		answer.append(next)

	if carry:
		print "No more scenarios"
		raise RuntimeError, "no more scenarios"

	answer.reverse()
	return tuple(answer)

def get_name(name, descr):
	return name+"_".join(list(descr))

def get_first_scen(name):
	descr=("low","low","low","low")
	return (get_name(name,descr),descr)

def get_next_scen(name,prev_descr):
	descr=get_next_descr(prev_descr)
	return (get_name(name,descr), descr)

ages=("mol1","mol2","per","prs","spl","srd")
spel_pers=("per","spl")
kinds=("101","102","103","104","105","124","125")

M_NAME="m_59"

def get_S(traj):
	return getattr(traj, M_NAME)

def make_attr(age, kind):
	return age+"_"+kind

def biomass(traj):
	conc=concentration(START_VOL, traj)
	#print conc
	S=get_S(traj)
	S=S.S * conc
	#print S - START_VOL.V
	#print S
	summ=0 # S.cult_101.__copy__()
	for age in ages:
		for kind in kinds:
			summ+=getattr(S, make_attr(age, kind))
	return ModelParameter({"V":summ})

def free_area(traj):
	S=get_S(traj)
	S=S.S
	return ModelParameter({"S":S.no_forest___None__})

def spel_per(traj):
	S=get_S(traj)
	S=S.S
	summ=0 #zeros(51)#S.cult_101.__copy__()
	for age in spel_pers:
		for kind in kinds:
			summ+=getattr(S, make_attr(age, kind))
	return ModelParameter({"S":summ})

def write_data(file, varname, descr, traj):
	name=get_name(varname, descr)
	traj.save(file, name)
	# graphics

def coexistance(traj):
	traj=traj.Copy()
	S=get_S(traj)
	S=S.S
	#summ=S.cult_101.__copy__()
	for kind in kinds:
		all=0.0
		if kind=="101":
			all+=S.cult_101
		for age in ages:
			all+=getattr(S, make_attr(age, kind))
		if kind=="101":
			S.cult_101 /= all
		for age in ages:
			setattr(S, make_attr(age, kind), getattr(S, make_attr(age, kind)) / all)
	del S.no_forest___None__
	return traj

def logging_vol(traj):
	conc=concentration(START_VOL, traj)
	traj=traj.Copy()
	S=get_S(traj)
	S=S.S
	rc=ModelParameter()
	for kind in kinds:
		attr=make_attr("logged", kind)
		spl=make_attr("spl", kind)
		per=make_attr("per", kind)
		if kind<>"105": # 105 is not loggable
			logged=getattr(S, attr)
			sspl=getattr(S, spl)
			sper=getattr(S, per)
			s=sspl+sper
			cspl=getattr(conc, spl)
			cper=getattr(conc, per)
			v=logged*((cspl * sspl / s) + (cper * sper / s))
			#v=logged * cspl
			setattr(rc, attr, v)
	return ModelParameter({"V":rc})

def open1(name, mode):
	f=open(name, mode)
	f.write("from mutils import *\nfrom Numeric import *\n")
	return f

def main():
	pname="scen_"
	n=1
	(name, descr)=get_first_scen(pname)
	fbio=open1("biomass.py","w")
	ffree=open1("free.py","w")
	fsp=open1("spelper.py","w")
	#fco=open1("coex.py","w")
	flog=open1("logging.py","w")
	while n:
		print name
		val=globals()[name]
		write_data(fbio, "biomass_", descr, biomass(val))
		write_data(ffree, "free_", descr, free_area(val))
		write_data(fsp, "spelper_", descr, spel_per(val))
		#write_data(fco, "coex_", descr, coexistance(val))
		write_data(flog, "logging_", descr, logging_vol(val))

		try:
			(name, descr)=get_next_scen(pname, descr)
		except RuntimeError:
			n=0
	fbio.close()
	fsp.close()
	ffree.close()
	#fco.close()
	flog.close()

if __name__=="__main__":
	main()



from quant import *
import types
import math
from Numeric import *
from mutils import *

DEBUG=0
if DEBUG:
	DEBUG_INSTALL=1
else:
	DEBUG_INSTALL=0

MODEL_TIME=50

if DEBUG<10:
	import mx.ODBC.Windows as ODBC
	import dme


class ForestModel:
	def __init__(self, tree_names, **keyargs):
		self.tree_names=tree_names
		self.options={}
		for k,v in keyargs.items():
			self.options[k]=v
	def install_all(self, region, t0):
		parts=[
			self.install_age_classes(),
			self.user_install(),
		]
		self.install_queries()
		self.install_basic_consts(t0)
		self.install_reaction_proc()
		self.install_basic_kb()
		parts.append(self.install_tree_kinds(region))
		if not self.options.get("NO_CULTURES", None):
			parts.append(self.install_cultures(region))
			parts.append("culture_kind(f101)")
		if not self.options.get("NO_LOGGING", None):
			parts.append(self.install_logging(region))
		if not self.options.get("NO_FIRES", None):
			parts.append(self.install_fires(region))
		parts.append(self.install_task(region, t0))
		s=""
		for part in parts:
			part=part.strip()
			if len(part)>0:
				s=s+","+part
		if len(s)>0:
			s=s[1:]
		self.install_KB(s)
	def user_install(self):
		return ""
	def install_tree_kinds(self, region_name):
		#install_predicate("tree_kind", 1)
		s=""
		for tree_name in self.tree_names:
			try:
				install_constant("f"+str(tree_name), tree_name)
			except ValueError:
				pass
			s+="tree_kind(%s,f%i) " % (region_name, tree_name)
			if tree_name!=105: # no cedra
				# Can be logged
				s+="loggable(f%i) " % tree_name
		s=s.strip().replace(" ",",")
		return s
	def install_age_classes(self):
		try:
			self._age_classes_installed
			return ""
		except AttributeError:
			pass
		install_predicate("age_class", 1)
		s=""
		for class_name in ["cult","mol1","mol2","srd","prs", "spl", "per"]:
			install_constant(class_name, class_name)
			s+="age_class(%s) " % class_name
		s=s.strip().replace(" ",",")+",after(mol1,mol2),after(mol2,srd),after(srd,prs),after(prs,spl),after(spl,per)"
		s+=",loggable_age(spl),loggable_age(per)"
		install_constant("no_forest", "no_forest")
		self._age_classes_installed=1
		return s
	def install_basic_consts(self, t0):
		try:
			self._basic_constants_installed
			return ""
		except AttributeError:
			pass
		for name in ["model_dyn", "ten_years", "forest", "forest_res", "landscape",
			"yes", "no", "forest_cultures","all","forest_fires","all_logging","forest_logging"]:
			install_constant(name, name)
		install_constant("None", '__None__')
		install_constant("logged", 'logged')
		install_constant("t0",t0)
		install_constant("nil",0.0)
		install_constant("one",1.0)
		self._basic_constants_installed=1
	def install_reaction_proc(self):
		try:
			self._reaction_proc_installed
			return ""
		except AttributeError:
			pass
		install_predicate("State",5,self.reaction_proc_state)
		install_predicate("Trans",4,self.reaction_proc_trans)
		install_predicate("Print",1,self.reaction_proc_print)
		install_predicate("DefineModel",2,self.reaction_proc_define_model)
		self._reaction_proc_installed=1
		return ""
	def install_basic_kb(self):
		translate_string("""
fm ModelDeterm()=\
  a:a,t0[Problem(a, model_dyn), Prognose(a, ten_years, t0), \
    Geosystem_Range(a, landscape), Landscape_Type(a, forest) \
     > \
      e:m,s0[ModelType(m, a, forest_res, t0), DefineModel(m,a), State(m, s0, no_forest, \
		square(a,None, no_forest, t0), None)]];
fm NoToMol()=\
  a:m,a,k,t0,s0,sq [\
    ModelType(m,a,forest_res,t0),\
	tree_kind(a,k),age_class(mol1),\
	State(m,s0,no_forest,sq,None)\
      >\
    e:s1[State(m,s1,mol1,square(a,k,mol1,t0),k), \
	    Trans(m,s0,s1,trans_time(no_forest,mol1,k,t0))\
	]\
  ];
fm Cultures()=\
	a:ck,m,a,t0,s1,s0,sq0[\
	  ModelType(m,a,forest_res,t0),\
	  State(m,s0,no_forest,sq0,None),\
	  State(m,s1,mol1,square(a,ck,mol1,t0),ck),\
	  TakesPlace(a,forest_cultures,yes),culture_kind(ck)\
	  >\
	  e: sk[\
		State(m,sk,cult,square(a,ck,cult,t0),ck),\
		Trans(m,sk,s1,trans_time(cult,mol1,ck,t0)),\
		Trans(m,s0,sk,CalcCulturesTrans(square(a,ck,cult,t0),sq0))\
	  ]\
	]
fm FiresDef()=\
	a:m,a,t0[\
	  Problem(a,forest_fires),\
	  ModelType(m,a,forest_res,t0)\
	  >\
	  FiresBaseIntensity(m,a,\
			div(\
				CalcBaseFiresIntensity(a,t0), square(a,all,all,t0)\
				)\
	  )\
	]
fm FiresProp()=\
	a:m,a,t0,s,s0,name,sq,sq_no_forest,fi,k[\
	  ModelType(m,a,forest_res,t0),\
	  State(m,s,name,sq,k),age_class(name),\
	  State(m,s0,no_forest,sq_no_forest, None),\
	  FiresBaseIntensity(m,a,fi)\
	  >\
	  Trans(m,s,s0,fi)\
	]
fm Fires()=FiresDef()&FiresProp()
fm LoggingDef()=\
	a:m,a,t0[\
	  Problem(a,forest_logging),\
	  ModelType(m,a,forest_res,t0)\
	  >\
	  LoggingIntensity(m,a,\
			div(\
				CalcLoggingIntensity(a,t0), square(a,all_logging,all_logging,t0)\
				)\
		) &\
			a:k,s0,sq_no_forest[loggable(k),\
					State(m,s0,no_forest,sq_no_forest, None)\
				>\
				e:sl[\
					State(m,sl,logged,nil,k),\
					Trans(m,sl,s0,one)\
				]\
			]\
	]
fm LoggingProp()=\
	a:m,a,t0,s,s0,name,k,sq_no_forest,li,sl,sq_logged[\
	  ModelType(m,a,forest_res,t0),\
	  State(m,s,name,square(a,k,name,t0),k),age_class(name),loggable_age(name),\
	  State(m,s0,no_forest,sq_no_forest,None),\
	  State(m,sl,logged,sq_logged,k),\
	  loggable(k),\
	  LoggingIntensity(m,a,li)\
	  >\
	  Trans(m,s,sl,li) & \
	  (\
	    Problem(a,forest_fires) \
		> \
		Trans(m,s,s0,mul(li, FireLogCf()))\
	  )\
	]
fm Logging()=LoggingDef(),LoggingProp()
fm AToB()=\
  a:m,t0,a,k,cl,cl1,s [\
    ModelType(m,a,forest_res,t0),tree_kind(a,k), age_class(cl),after(cl,cl1),\
	State(m,s,cl,square(a,k,cl,t0),k)\
      >\
    e:s1[State(m,s1,cl1,square(a,k,cl1,t0),k),Trans(m,s,s1,trans_time(cl,cl1,k,t0))] \
  ];
fm BaseBuilding()=\
  ModelDeterm() & NoToMol() & Cultures() & AToB() & Fires() & Logging();
set steps_count 100000;
#set trace 16;
#set tree_write_depth 60;
set debug_level 0;
""")
	def install_task(self, region_name, t0):
		install_constant(region_name,region_name)
		s="Problem(%s,model_dyn),Prognose(%s,ten_years,t0),Geosystem_Range(%s,landscape),Landscape_Type(%s,forest)" % (region_name,region_name,region_name,region_name)
		return s
	def install_cultures(self, region_name):
		s="TakesPlace(%s,forest_cultures,yes)" % (region_name)
		return s
	def install_logging(self, region_name):
		s="Problem(%s,forest_logging)" % region_name
		return s
	def install_fires(self, region_name):
		s="Problem(%s,forest_fires)" % region_name
		return s
	def install_KB(self, data_str):
		s="fm KB=%s & BaseBuilding()" % data_str
		print s
		translate_string(s)
	def _div(self, a,b):
		return a/b
	def _mul(self, a,b):
		return a*b
	def install_queries(self):
		install_function("square", 4, self.proc_query_square)
		install_function("trans_time", 4, self.proc_query_trans_time)
		install_function("div", 2, self._div)
		install_function("mul", 2, self._mul)
		install_function("CalcBaseFiresIntensity", 2, self.proc_query_fires_intensity)
		install_function("CalcLoggingIntensity", 2, self.proc_query_logging_square)
		install_function("CalcCulturesTrans", 2, self.proc_query_cultures_trans)
		install_function("FireLogCf", 0, self.proc_query_logging_fires_coefficient) # %was% area, year
		return ""
	def proc_query_square(self, region, kind, age, time_moment):
		if kind=='__None__':
			print "SELECT square FROM data WHERE kind='EMPTY' AND age=%s AND date=%i" % (age, time_moment)
		else:
			print "SELECT square FROM data WHERE kind=f%i AND age=%s AND date=%i" % (kind, age, time_moment)
		return 100
	def proc_query_trans_time(self, age1, age2, kind, time_moment):
		print "SELECT trans_time FROM data WHERE kind=f%i AND age=%s" % (kind, age2)
		return 1/20.0
	def reaction_proc_state(self, model, state_name, state_descr, square, kind):
		print "Installing state:", state_name, "of model", model, "descr:", state_descr, "sq:", square, "kind:", kind
	def reaction_proc_trans(self, model, begin,end, value):
		print "Installing trans of model", model, "from:", begin, "to:", end, "val:", value
	def reaction_proc_print(self, arg1):
		print "--> Printed:", arg1
	def reaction_proc_define_model(self, model_name, region_name):
		print "Defined %s as %s" % (model_name, region_name)
	def proc_query_logging_fires_coefficient(self): #, area, time_moment):
		return 0.88
	def infer(self):
		f=get_descriptor("KB",0)
		if f:
			return f.prove(trace=self.infer_trace_proc)
		else:
			raise ValueError, "cannot prove None object"
	def infer_trace_proc(self, pcf):
		global trace_no
		trace_no +=1
		print "trace--------- N", trace_no
		#pcf.print_as_tree()
		return 1
	def DoModel(self, *args):
		self.install_all(*args)
		(rc, res)=self.infer()
		bases=res.get_reduced_bases()
		fm=ForestModeller(self, starttime=0.0, endtime=MODEL_TIME)

		fm.Model()
		print "model done"
		self.conn.close()
		traj=fm.trajectories
		del fm
		return traj


#===========================================

trace_no=0

ForestKind={
	101:1,
	102:1,
	103:1,
	104:1,
	105:1,
	124:1,
	125:1,
	126:1,
	131:1,
	132:1,
}

class ExcelForestModel(ForestModel):
	def __init__(self, dsn, year=1973, tree_names=[], **keyargs):
		global ForestKind
		self.dsn=dsn
		self.year=year
		if DEBUG<10:
			self.conn=ODBC.connect(dsn, user="", password="", clear_auto_commit=0)
			cursor=self.conn.cursor()
			cursor.execute('select Kind from "S%s$"' % year)
			fks=cursor.fetchall()
			if fks is None:
				raise ValueError, "wrong year is given (returned empty set)"
			tree_names=map(lambda (x,): int(x), fks)
		else:
			tree_names=[101,102,103,104,105,124,125]
		#tree_names=[101]
		if DEBUG:
			print "tree_names:", tree_names
		#raw_input("Connect debugger")
		ForestModel.__init__(self, tree_names, **keyargs)

		self.options["culture_kind"]=101 # i.e. Sosna

	def get_cultures(self, kind, age, time_moment):
		cursor=self.conn.cursor()
		cursor.execute('select year,dSk from "cultures$"')
		data=cursor.fetchone()
		Lk1=self.proc_query_trans_time("cult", "mol1", kind, time_moment)
		S=0
		year_last=int(time_moment)
		phase=1
		self.USk={}
		while data:
			y=int(data[0])
			dS=data[1]
			dS=dS/1000.0 # to make thousents of ga (kga)
			if y>year_last:
				phase=3
			if phase==1:
				y_p=y
				phase=2
				S=dS
			elif phase==2:
				y_t=y
				dy=y_t-y_p
				S = S - S * (Lk1**dy) + dS	#dSk
				y_p=y_t
			else:
				pass
			self.USk[y]=dS
			#print data[0], S, data[1]
			data=cursor.fetchone()
		if DEBUG:
			print "Summary of Sk (kga):", S, kind, age
		return S
		"""
		try:
			self.options["NO_CULTURES"]
			self.Sk0=0.0
			self.USk[year_last]=0.0
			print "Warning: No cultures!"
		except KeyError:
			self.Sk0=S
		del S,F101
		"""
	def get_all_square(self, region, time_moment):
		cursor=self.conn.cursor()
		cursor.execute('select S1,S2,Ssr,Spr,SIspper from "S%s$"' % time_moment)
		data=cursor.fetchone()
		s=0.0
		while data:
			s+=reduce(lambda x,y: x+y, data, 0)
			data=cursor.fetchone()
		# without cultures
		if DEBUG:
			print "all the square:", s
		return s

	def get_all_loggable_square(self, region, time_moment):
		cursor=self.conn.cursor()
		cursor.execute('select kind, SIspper from "S%s$"' % time_moment)
		data=cursor.fetchone()
		s=0.0
		while data:
			if int(data[0])!=105: # Cedra is not logged
				#print ":", data, s
				s+=data[1]
			data=cursor.fetchone()
		# without cultures
		if DEBUG:
			print "all the loggable square:", s
		return s

	def proc_query_logging_square(self, region, time_moment):
		LI=self.options.get("LOGGING_INTENSITY",None)
		if LI is not None:
			if DEBUG:
				print "all the loggable square (intens base) LI:", LI
			return LI
		cursor=self.conn.cursor()
		cursor.execute('select Year,S from "logging$"')
		data=cursor.fetchone()
		while data:
			if data[0]>=time_moment:
				s=data[1]/1000.0
				if DEBUG:
					print "all the loggable square (intens base):", s
				return s
			data=cursor.fetchone()
		raise SystemError, "cannot find logging data"

	def proc_query_square(self, region, kind, age, time_moment):
		trt=\
			{
				"mol1":"1",
				"mol2":"2",
				"srd":"sr",
				"prs":"pr",
				"per":"per",
				"spl":"Ispper", # But SIspper = sp+per !!!!!
			}
		if age=="cult":
			return self.get_cultures(kind, age, time_moment)
		if age=="all":
			return self.get_all_square("Irkutsk",time_moment)
		if age=="all_logging":
			return self.get_all_loggable_square("Irkutsk",time_moment)
		age=trt.get(age, age)
		if kind=='__None__':
			query='SELECT S0 FROM "free%s$"' % (time_moment,)
		else:
			query='SELECT S%s FROM "S%s$" WHERE Kind=%s' % \
				(age, time_moment, kind)
		cursor=self.conn.cursor()
		#print query
		try:
			_a=1
			cursor.execute(query)
			_a=0
		finally:
			if _a:
				print "Query was:", query

		(s,)=cursor.fetchone()
		if age=="Ispper":
			per=self.proc_query_square(region, kind, "per", time_moment)
			s=s-per
		#print query
		#print s
		return s
	def proc_query_trans_time(self, age1, age2, kind, time_moment):
		tr=\
			{
				"no_forest":"0",
				"cult":"k",
				"mol1":"1",
				"mol2":"2",
				"srd":"3",
				"prs":"4",
				"per":"6",
				"spl":"5",
			}
		age1=tr.get(age1,age1)
		age2=tr.get(age2,age2)
		query='SELECT L%s%s FROM "grow%s$" WHERE kind=%i' % (age1, age2, time_moment, kind)
		cursor=self.conn.cursor()
		#print query
		cursor.execute(query)
		(l,)=cursor.fetchone()
		return l

	def proc_query_cultures_trans(self, cultures_area, empty_area):
		CI=self.options.get("CULTURES_INTENSITY",None)
		if CI is None:
			return cultures_area / empty_area
		else:
			return CI / empty_area

	def proc_query_fires_intensity(self, area, time_moment):
		FI=self.options.get("FIRES_INTENSITY", None)
		if FI is not None:
			return FI*(1+self.options.get("ILL_PERCENT", 0))
		cursor=self.conn.cursor()
		cursor.execute('select Ssx, N, L, KdNsx FROM "people%s$"' % time_moment)
		(Ssx, N, L, KdNsx) = cursor.fetchone()
		cursor.execute('select KULogging, KfN, KfL, KfSsx, dUf from "fires%s$"' % time_moment)
		(KULogging, KfN, KfL, KfSsx, dUf) = cursor.fetchone()
		#antropo_fires=KfN * N + KfL * L + KfSsx * Ssx + dUf
		antropo_fires=15.013
		#print "Antropo_fires=", antropo_fires
		return antropo_fires
	def reaction_proc_state(self, model, state_name, state_descr, square, kind):
		if DEBUG_INSTALL:
			print "Installing state:", state_name, "of model", model, "descr:", state_descr, "sq:", square, "kind:", kind
		try:
			m=getattr(self, str(model))
		except AttributeError:
			setattr(self, str(model), ModelParameter())
			self.model_name=str(model)
			m=getattr(self, self.model_name)
			self.chain_head=None
		self.chain_head=dmi=dme.DMItem(square, self.chain_head)
		setattr(m, str(state_name), (dmi, (str(state_descr), str(kind))))
		#setattr(m, "_"+str(state_name)+"_descr", (str(state_descr), str(kind)))
		try:
			m._states
		except AttributeError:
			m._states={}
		m._states[(str(state_descr), str(kind))]=str(state_name)
	def reaction_proc_define_model(self, model_name, region_name):
		#region_name="m_"+region_name
		print "-->Defined %s as %s" % (model_name, region_name)
		try:
			self.model_names
		except AttributeError:
			self.model_names={}
		try:
			self.region_names
		except AttributeError:
			self.region_names={}
		model_name=str(model_name)
		region_name=region_name
		self.model_names[model_name]=region_name
		self.region_names[region_name]=model_name
		#print "Definition:", self.model_names
	def reaction_proc_trans(self, model, begin,end, value):
		if DEBUG_INSTALL:
			print "Installing trans of model", model, "from:", begin, "to:", end, "val:", value
		m=getattr(self, str(model))
		b=getattr(m, str(begin))
		e=getattr(m, str(end))
		dme.DMConnect(b[0], e[0], value)
		#print "Definition:", self.model_names
	def variable(self, name, pool_name=None):
		try:
			dic=self.variables
		except AttributeError:
			dic=self.variables={}
		if pool_name:
			dic=self.variables[pool_name] # raise error instead creating new one
		try:
			return dic[name]
		except KeyError:
			dic[name]=None
			return dic[name]
	def prepare_storage(self, obj):
		def convert(value, obj):
			return obj.__copy__()
		def convert_name(value, key, obj):
			return "_".join(value[1])
		m=getattr(self, self.model_name)
		S=m.ConvertWithNames(convert, convert_name, obj)
		return ModelParameter({self.model_name:ModelParameter({"S":S})})
	def diff(self, model2, trajectories1, trajectories2, result):
		def leaf(name, value, ((t1,t2,res), m_name, (m1,m2))):
			(name,)=name # as we know that the trajectories and model have 1 lavel.
			if name[0]=="_":
				return
			#print "-->", t1.__dict__
			m=getattr(t1, m_name).S
			a=getattr(m, name)
			md=getattr(m1, m_name)
			sdescr=getattr(md, "_"+name+"_descr")
			region_name=m1.model_names[m_name]
			m_name2=m2.region_names[region_name]
			mm2=getattr(t2, m_name2).S
			md2=getattr(m2, m_name2)
			r=getattr(res, m_name).S
			r=getattr(r, name)
			try:
				state_name=md2._states[sdescr]
			except KeyError:
				for number in range(len(r)):
					r[number]=-1e39
				return
			a2=getattr(mm2, state_name)
			for number in range(len(r)):
				r[number]=a[number]-a2[number]
		for model_name, model_descr in self.model_names.items():
			m=getattr(self, model_name)
			m.ForEachDo(leaf, ((trajectories1,trajectories2, result), model_name, (self,model2)))

BD_NAME="bd_les"
"""
P1	Доля спелых и перестойных лесов в лесной площади, %;
-----------------------------
P2	Доля спелых и перестойных лесов в покрытой лесной растительностью площади, %;
P3	Средний запас древесины в спелых и перестойных лесах, кбм/га;
P4	Средний запас древесины в спелых и перестойных лесах, возможных для эксплуатации, кбм/га;
P5	Средний запас древесины в спелых и перестойных хвойных лесах, возможных для эксплуатации, кбм/га;
P6	Доля спелых и перестойных лесов, возможных для эксплуатации, в покрытой лесной растительностью площади, %;
P7	Доля спелых и перестойных хвойных лесов, возможных для эксплуатации, в покрытой лесной растительностью площади, %;
P8	Средний запас древесины  на землях, покрытых лесной растительностью, кбм/га;
P9	Доля хвойных лесов в покрытой лесной растительностью площади, %;
-----------------------------
P10	Доля сосняков в покрытой лесной растительностью площади, %;
P11	Доля ельников в покрытой лесной растительностью площади, %;
P12	Доля пихтарников в покрытой лесной растительностью площади, %;
P13	Доля лиственничников в покрытой лесной растительностью площади, %;
P14	Доля кедровников в покрытой лесной растительностью площади, %;

P15	Доля березняков в покрытой лесной растительностью площади, %;
P16	Доля осинников в покрытой лесной растительностью площади, %;
-----------------------------
P17	Средний возраст лесов, лет;
P18	Средний возраст хвойных лесов, лет;
P19	Средний возраст лиственных лесов, лет;
P20	Доля лесов искусственного происхождения;
-----------------------------
P21	Фонд лесовосстановления (в процентах к лесным землям);
-----------------------------
P22	Доля лесов первой группы, %;
P23	Доля лесов второй группы, %;
P24	Доля лесов третьей группы, %;
P25	Доля лесов, возможных для эксплуатации;
P26	Доля хвойных молодняков в возрасте до 20 лет, %;
----------------------------------------------------------------
P27	Общий средний прирост, кбм/га;
P28	Средний годичный прирост древесины хвойных пород, кбм/га;
P29	Средний годичный прирост древесины мягколиственных пород, кбм/га;
P30	Средний бонитет хвойных лесов;
P31	Средний бонитет лиственных лесов;
P32	Средняя полнота хвойных лесов;
P33	Средняя полнота лиственных лесов;
P34	Давность лесоустройства;
P35	Общий размер отпуска ликвидной древесины с 1 га покрытых лесной растительностью земель, кбм.
"""
# Доля молодников лиственных берется в 2 раза больше и отнимается от доли всех лиственных
FOREST_KIND=[101,102,103,104,105,124,125]
"""
	101:"СОСНА",
	102:"ЕЛЬ",
	103:"ПИХТА",
	104:"ЛИСТВЕННИЦА",
	105:"КЕДР",
	107:"ИТОГО ХВОЙНЫХ",
	124:"БЕРЕЗА",
	125:"ОСИНА",
	126:"ОЛЬХА СЕРАЯ",
	131:"ТОПОЛЬ",
	132:"ИВЫ ДРЕВОВИДНЫЕ",
	133:"ИТОГО МЯГКОЛИСТВЕННЫХ",
"""

class GISForestModel(ForestModel):
	def __init__(self, dsn, regions, last_year=2050, tree_names=[], **keyargs):
		global ForestKind
		self.dsn=dsn
		self.last_year=last_year
		tree_names=FOREST_KIND
		self.regions=regions
		self.conn=ODBC.connect(dsn=dsn, user="", password="")

		ForestModel.__init__(self, tree_names, **keyargs)

		self.options["culture_kind"]=101 # i.e. Sosna

	def _div(self, a,b):
		return a
	def get_value(self, region, *args):
		s=reduce(lambda x,y: x+","+y, args)
		cursor=self.conn.cursor()
		query="select %s from bd_les_o where LESX='%s'" % (s, region[3:])
		#print query
		cursor.execute(query)
		data=cursor.fetchone()
		if len(data)==1:
			return data[0]
		return data
	def get_cultures(self, region, kind, age, time_moment):
		return self.get_value(region, "P20")
	def get_all_square(self, region, time_moment):
		return 100.0
	def get_all_loggable_square(self, region, time_moment):
		return self.get_value(region, "P6")
	def proc_query_logging_square(self, region, time_moment):
		return 0.007
	def proc_query_square(self, region, kind, age, time_moment):
		trt={
			101:"P10",
			102:"P11",
			103:"P12",
			104:"P13",
			105:"P14",
			124:"P15",
			125:"P16",
		}
		if age=="cult":
			return self.get_cultures(region, kind, age, time_moment)
		if age=="all":
			return self.get_all_square(region, time_moment)
		if age=="all_logging":
			return self.get_all_loggable_square(region, time_moment)
		if kind=='__None__':
			return self.get_value(region,"P21")
		name=trt[kind]
		s=self.get_value(region, name)
		m=self.get_value(region,"P26")
		if kind in [101, 102, 103, 104, 105]:
			if age=="mol1":
				s=m*s/100.0
			elif age=="per":
				s=(100-m)*s/100.0
		else:
			m*=1.2	# empirical
			if m>100:
				m=100
			if age=="mol1":
				s=m*s/100.0
			elif age=="per":
				s=(100-m)*s/100.0
		#print "Square ->>> :", s, m, kind, age
		return s
		#raise "Untrearted combination %s %s" % (repr(kind), repr(age))

	def proc_query_trans_time(self, age1, age2, kind, time_moment):
		tbl={
			"no_forest":(1/20.0),
			"mol1":(1/(4*20.0)),
			"cult":(0.07518797)
		}
		base=tbl[age1]
		if kind in [101, 102, 103, 104]:
			return base
		elif kind==105:
			return base/2
		else:
			return base*2
	def proc_query_fires_intensity(self, area, time_moment):
		return 0.002 # Antropofires
	def reaction_proc_define_model(self, model_name, region_name):
		#print "Defined %s as %s" % (model_name, region_name)
		try:
			self.model_names
		except AttributeError:
			self.model_names={}
		try:
			self.region_names
		except AttributeError:
			self.region_names={}
		model_name=str(model_name)
		region_name=int(region_name[3:])
		self.model_names[model_name]=region_name
		self.region_names[region_name]=model_name
		#print "Definition:", self.model_names
	def reaction_proc_state(self, model, state_name, state_descr, square, kind):
		#print "Installing state:", state_name, "of model", model, "descr:", state_descr, "sq:", square
		model_name=str(model)
		try:
			m=getattr(self, model_name)
		except AttributeError:
			setattr(self, model_name, ModelParameter())
			m=getattr(self, model_name)
			try:
				self.chain_head
			except AttributeError:
				self.chain_head=None
		self.chain_head=dmi=dme.DMItem(square, self.chain_head)
		setattr(m, str(state_name), dmi)
		setattr(m, "_"+str(state_name)+"_descr", (str(state_descr), str(kind)))
		try:
			m._states
		except AttributeError:
			m._states={}
		m._states[(str(state_descr), str(kind))]=str(state_name)
	def reaction_proc_trans(self, model, begin,end, value):
		#print "Installing trans of model", model, "from:", begin, "to:", end, "val:", value
		m=getattr(self, str(model))
		b=getattr(m, str(begin))
		e=getattr(m, str(end))
		dme.DMConnect(b, e, value)
	def prepare_storage(self, obj):
		def convert(value, obj):
			return obj.__copy__()
		rc={}
		for model_name in self.model_names.keys():
			m=getattr(self, model_name)
			S=m.Convert(convert,obj)
			rc[model_name]=ModelParameter({"S":S})
		return ModelParameter(rc)
	def install_age_classes(self):
		try:
			self._age_classes_installed
			return ""
		except AttributeError:
			pass
		install_predicate("age_class", 1)
		s=""
		for class_name in ["cult","mol1","per"]:
			install_constant(class_name, class_name)
			s+="age_class(%s) " % class_name
		s=s.strip().replace(" ",",")+",after(mol1,per),loggable_age(per)"
		install_constant("no_forest", "no_forest")
		self._age_classes_installed=1
		return s
	def install_queries(self):
		install_function("square", 4, self.proc_query_square)
		install_function("trans_time", 4, self.proc_query_trans_time)
		install_function("div", 2, self._div)
		install_function("mul", 2, self._mul)
		install_function("CalcBaseFiresIntensity", 2, self.proc_query_fires_intensity)
		install_function("CalcLoggingIntensity", 2, self.proc_query_logging_square)
		install_function("FireLogCf", 0, self.proc_query_logging_fires_coefficient) # %was% area, year
		return ""
	def install_all(self):
		parts=[
			self.install_age_classes(),
			self.user_install(),
		]
		self.install_queries()
		self.install_basic_consts(0)
		self.install_reaction_proc()
		self.install_basic_kb()
		for region in self.regions:
			region="reg%i" % region
			parts.append(self.install_tree_kinds(region))
			if not self.options.get("NO_CULTURES", None):
				parts.append(self.install_cultures(region))
				parts.append("culture_kind(f101)")
			if not self.options.get("NO_LOGGING", None):
				parts.append(self.install_logging(region))
			if not self.options.get("NO_FIRES", None):
				parts.append(self.install_fires(region))
			parts.append(self.install_task(region, 0))
		#s=s.strip().replace(" ",",")
		s=""
		for part in parts:
			part=part.strip()
			if len(part)>0:
				s=s+","+part
		if len(s)>0:
			s=s[1:]
		self.install_KB(s)
	def diff(self, model2, trajectories1, trajectories2, result):
		def leaf(name, value, ((t1,t2,res), m_name, (m1,m2))):
			(name,)=name # as we know that the trajectories and model have 1 lavel.
			if name[0]=="_":
				return
			#print "-->", t1.__dict__
			m=getattr(t1, m_name).S
			a=getattr(m, name)
			md=getattr(m1, m_name)
			sdescr=getattr(md, "_"+name+"_descr")
			region_name=m1.model_names[m_name]
			m_name2=m2.region_names[region_name]
			mm2=getattr(t2, m_name2).S
			md2=getattr(m2, m_name2)
			r=getattr(res, m_name).S
			r=getattr(r, name)
			try:
				state_name=md2._states[sdescr]
			except KeyError:
				for number in range(len(r)):
					r[number]=-1e39
				return
			a2=getattr(mm2, state_name)
			for number in range(len(r)):
				r[number]=a[number]-a2[number]
		for model_name, model_descr in self.model_names.items():
			m=getattr(self, model_name)
			m.ForEachDo(leaf, ((trajectories1,trajectories2, result), model_name, (self,model2)))

class ManyGISForestModel(GISForestModel):
	def __init__(self, dsn, **keyargs):
		conn=ODBC.connect(dsn=dsn, user="", password="")
		c=conn.cursor()
		c.execute("select P34, LESX from bd_les_o")
		all=c.fetchall()
		n=[]
		for (year, name) in all:
			try:
				if int(year)>0:
					n.append(int(name))
			except ValueError:
				pass
		#print n
		#n=[11252218,11252207]
		#n=[11252218]
		GISForestModel.__init__(self, dsn, n, **keyargs)
	def print_something(self, number, trajectories):
		def leaf(name, value, (parameter, m_name, number, model, descr, cursor)):
			(name,)=name # as we know that the trajectories and model have 1 lavel.
			if name[0]=="_":
				return
			m=getattr(parameter, m_name).S
			a=getattr(m, name)
			md=getattr(model, m_name)
			sdescr=getattr(md, "_"+name+"_descr")
			#print name, getattr(md, "_"+name+"_descr"), descr, a[number]
			#a[number]=value.value
			if sdescr[0]=='no_forest':
				query="update bd_les set P21=%s where LESX='%i'" % (a[number], descr)
			elif sdescr[0]=='cult':
				query="update bd_les set P20=%s where LESX='%i'" % (a[number], descr)
			elif sdescr[0]=='mol1' and int(sdescr[1]) in [101,102,103,104,105]:
				query="update bd_les set P26=P26+%s where LESX='%i'" % (a[number], descr)
			else:
				tr={
					101:"P10",
					102:"P11",
					103:"P12",
					104:"P13",
					105:"P14",
					124:"P15",
					125:"P16",
				}
				field_name=tr[int(sdescr[1])]
				query="update bd_les set %s=%s+%s where LESX='%i'" % (field_name, field_name, a[number], descr)
			#print query
			cursor.execute(query)
		for model_name, model_descr in self.model_names.items():
			m=getattr(self, model_name)
			cursor=self.conn.cursor()
			cursor.execute("update bd_les set P10=0, P11=0, P12=0, P13=0, P14=0, P15=0, P16=0, P26=0  where LESX='%i'" % model_descr)
			m.ForEachDo(leaf, (trajectories, model_name, number, self, model_descr, cursor))
			cursor.execute("update bd_les set P1=P10+P11+P12+P13+P14+P15+P16 where LESX='%i'" % model_descr)
			self.conn.commit()

def SpecialName(s):
	if s=="":
		return 1
	if s[0]=="_":
		if len(s)>1 and s[1]!="_":
			return 1
		else:
			return 0
		return 1
	return 0

"""
biggles_colors={
			#0xRRGGBB
	"red": 0x0f
	"green": 0x
	"blue": 0x
	"": 0x
	"": 0x
	"": 0x
	"": 0x
	"": 0x
	"": 0x
}
"""

biggles_point_types=[
	"none",
	"dot",
	"plus",
	"asterisk",
	"circle",
	"cross",
	"square",
	"triangle",
	"diamond",
	"star",
	"inverted triangle",
	"starburst",
	"fancy plus",
	"fancy cross",
	"fancy square",
	"fancy diamond",
]

biggles_line_types=[
	"solid",			"dotted",
	"dotdashed",		"shortdashed",
	"longdashed",	"dotdotdashed",
	"dotdotdotdashed"
]

class ForestModeller:
	def __init__(self,
			forest_model,
			starttime=0.0,
			endtime=1.0,
			interval=1.0,
			subdivisions=1.0):
		self.forest_model=forest_model
		self.starttime=starttime
		self.endtime=endtime
		self.interval=interval
		self.subdivisions=subdivisions
		self.dt=self.interval/self.subdivisions
		#print "Values_to_store:", self.values_to_watch
		self.model_head=self.forest_model.chain_head
		self.Reset()

	def Reset(self):
		self.time_moment=self.starttime
		self.step_no=0
		#self.forest_model.Reset()
		#self.forest_model.AfterStates(self)
		self.prepareStorage()

	def prepareStorage(self):
		i=(self.endtime-self.starttime) / self.interval
		self.range=int(math.ceil(i))+1
		self.time=arange(self.range, typecode=Float)*self.interval + self.starttime
		self.trajectories=self.forest_model.prepare_storage(zeros(self.range,Float))
		#self.trajectories=ModelParameter()
		self.fill_in_values(0)

	def fill_in_values(self, number):
		def leaf(name, value, (trajectories, m_name, number)):
			(name,)=name # as we know that the trajectories and model have 1 lavel.
			if name[0]=="_":
				return
			descr="_".join(value[1])
			m=getattr(trajectories, m_name).S
			a=getattr(m, descr)
			a[number]=value[0].value
			#print "---", name, value.value
		for model_name in self.forest_model.model_names.keys():
			m=getattr(self.forest_model, model_name)
			m.ForEachDo(leaf, (self.trajectories, model_name, number))

	def StepInterval(self):
		for n in range(self.subdivisions):
			#self.forest_model.BeforeStates(self)
			self.model_head.step(self.dt, 1)
			#self.forest_model.AfterStates(self)
		self.step_no+=1
		return self.step_no

	def Model(self):
		while (self.StepInterval()<self.range):
			self.fill_in_values(self.step_no)
		self.trajectories.time=self.time

	def _makeLegends(self, gpm, legends):
		def _leaf_func(fields, value, parameter):
			(legends, time, gpm)=parameter
			name=fields[-1]
			if name!="time" and not SpecialName(name):
				title=".".join(fields)
				legends.append(gpm.Data(time, value, title=title))
		def _branch_func(fields, value, parameter):
			(legends, time, gpm)=parameter
			name=fields[-1]
			global Interpret
			if name[0:2]=="__":	# specail function
				result=Interpret(name, value)
				legends.append(gpm.Data(time, result, title=".".join(fields)))
				return 0
			else:
				return 1

		_legends=[]
		legends.ForEachDo(_leaf_func, (_legends, self.trajectories.time, gpm), branch_func=_branch_func)
		return _legends


	def Gnuplot(self, device, file=None, legends=None, initial_text=""):
		import Gnuplot
		gp=Gnuplot.Gnuplot(debug=0)
		gp("set terminal %s monochrome" % device)
		gp(initial_text)
		gp("set data style linespoints")
		if file is None:
			pass
		else:
			gp("set output '%s'" % file)
		if legends is None:
			legends=self.trajectories

		_legends=self._makeLegends(Gnuplot, legends)
		apply(gp.plot, _legends)
		gp.reset()
		del gp

colors=[
	0x000000,
	0x0000FF,
	0x00FF00,
	0x00FFFF,
	0xFF0000,
	0xFF00FF,
	0xFFFF00,
	0x000080,
	0x008000,
	0x008080,
	0x800000,
	0x800080,
	0x808000
]

def _makeBigglesLegends(gpm, legends, model):
	def _leaf_func(fields, value, (legends, time, gpm, model)):
		name=fields[-1]
		if name!="time" and not SpecialName(name):
			if value[0]<-1e37:
				return
			m=getattr(model, fields[0])
			try:
				(descr, kind)=getattr(m, "_"+name+"_descr")
				title=".".join([descr,kind])
			except AttributeError:
				title=".".join(fields)

			#n=len(legends) / 2
			n=len(legends) % (len(biggles_line_types)*len(colors))
			color=colors[n % len(colors)]
			#print color, type(color)
			line_type=biggles_line_types[n / len(colors)]
			#l=gpm.Curve(time, value, type=line_type, color=color, linewidth=3)
			l=(abs(value[-1]), time, value, line_type, color, 3, title)
			#l.label=title
			legends.append(l)
	def _branch_func(fields, value, (legends, time, gpm, model)):
		name=fields[-1]
		global Interpret
		if name[0:2]=="__":	# specail function
			result=Interpret(name, value)
			l=gpm.Data(time, result) # colors...
			l.title=".".join(fields)
			legends.append(l)
			return 0
		else:
			return 1

	_legends=[]
	legends.ForEachDo(_leaf_func, (_legends, legends.time, gpm, model), branch_func=_branch_func)
	return _legends

def Bigglesplot(device, file, legends, model, initial_text="", title=""):
	import biggles
	p=biggles.FramedPlot()
	#p(initial_text)
	#p("set data style linespoints")
	p.title=title
	p.xlabel=r'$t$'
	p.ylabel='Square'
	if legends is None:
		legends=self.trajectories

	_legends=_makeBigglesLegends(biggles, legends, model)
	_legends.sort()
	_legends.reverse()
	new=[]
	for l in _legends:
		(skip, time, value, line_type, color, width, title)=l
		ll=biggles.Curve(time, value, type=line_type, color=color, linewidth=width)
		ll.label=title
		p.add(ll)
		new.append(ll)
	scale=0.2
	k=biggles.PlotKey( 1.1, 1, new )
	p.add(k)
	if file:
		p.write_eps(file)
	else:
		p.show()
	del p

scen_data_begin=\
	(
		{"low":15.013, "middle":195.172, "high":375.332}, # fires
		{"low":1.230, "middle":19.356, "high":37.482}, # cultures
		{"low":63.300, "middle":166.657, "high":242.908}, # logging
		{"low":0.0, "middle":0.75, "high":1.50}, # illness
	)
	
scen_data=\
	(
		{"low":15.013, "lm":75.066, "ml":135.119, "middle":195.172, "high":375.332}, # fires
		{"low":1.230, "middle":19.356,"mh":25.398, "hm":31.440, "high":37.482}, # cultures
		{"low":63.300, "middle":166.657, "high":242.908}, # logging
		{"low":0.0, "middle":0.75, "high":1.50}, # illness
	)

def get_values(descr):
	answer=[]
	for i in range(len(scen_data)):
		d=scen_data[i]
		answer.append(d[descr[i]])
	return answer

next_data={
	"low":(0,"lm"),
	"lm":(0,"ml"),
	"ml":(0,"middle"),
	"middle":(0,"mh"),
	"mh":(0,"hm"),
	"hm":(0,"high"),
	"high":(1,"low"),
}

def filter_true(comb):
	return 1
	
def filter_simple(comb):
	for c in comb:
		if not c in ["low","middle","high"]:
			return 0
	return 1
	
def filter1(comb):
	(a,b,c,d)=comb
	if d!="low":
		return 0
	return 1

def get_next_descr(prev, filter):
	answer=[]
	p=list(prev)
	n=1
	
	while n:
		p.reverse()
		carry=1
		for d in p:
			if carry:
				(carry, next)=next_data[d]
			else:
				next=d
			answer.append(next)
		if carry:
			raise RuntimeError, "no more scenarios"
		answer.reverse()
		n=not filter(answer)
		if n:
			p=answer
			answer=[]
	
	return tuple(answer)

def get_first_scen(filter):
	descr=("low","low","low","low")
	while 1:
		while not filter(descr):
			descr=get_next_descr(descr, filter)
		try:
			return (get_values(descr),descr)
		except KeyError:
			pass
		descr=get_next_descr(descr, filter)

def get_next_scen(prev_descr, filter):
	descr=get_next_descr(prev_descr, filter)
	while 1:
		while not filter(descr):
			descr=get_next_descr(descr, filter)
		try:
			return (get_values(descr),descr)
		except KeyError:
			pass
		descr=get_next_descr(descr, filter)

def generate_scenario((FI,CI,LI,IP), file, (PN_FI,PN_CI,PN_LI,PN_IP)):
	year=1973
	#FI=0
	#CI=0
	#LI=6.33
	#IP=0
	Region="Irkutsk"
	quant_init()
	natural=ExcelForestModel(dsn="forest", year=year, NO_FIRES=0, NO_LOGGING=0, NO_CULTURES=0,
		FIRES_INTENSITY=FI, CULTURES_INTENSITY=CI, LOGGING_INTENSITY=LI, ILL_PERCENT=IP)
	t1=natural.DoModel(Region, year)
	quant_done()
	data_name="_".join([PN_FI,PN_CI,PN_LI,PN_IP])
	file_name=file.name+data_name
	Bigglesplot("postscript","%s.eps" % file_name, t1, natural)
	#print t1
	t1.save(file,"scen_%s" % data_name)

def main():
	(scen, descr)=get_first_scen(filter1)
	n=1
	f=open("scenarios.dta","w")
	while n:
		print (scen,descr)
		generate_scenario(scen, f, descr)
		#sqd
		#break #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		try:
			(scen,descr)=get_next_scen(descr, filter1)
		except RuntimeError:
			n=0
	f.close()

def main1():
	quant_init()
	f1=ManyGISForestModel("forest_gis", NO_FIRES=0, NO_LOGGING=0, NO_CULTURES=0)
	t1=f1.DoModel()
	quant_done()
	#quant_init()
	#f2=ManyGISForestModel("forest_gis", NO_FIRES=1, NO_LOGGING=1, NO_CULTURES=1)
	#t2=f2.DoModel()
	#quant_done()
	#r=t1.Copy()
	#f1.diff(f2, t1, t2, r)
	#Bigglesplot("postscript","test.eps", r)
	f1.print_something(50, t1)
	print t1

def main3():
	year=1973
	Region="Irkutsk"
	quant_init()
	natural=ExcelForestModel(dsn="forest", year=year, NO_FIRES=0, NO_LOGGING=1, NO_CULTURES=1)
	t1=natural.DoModel(Region, year)
	Bigglesplot("postscript","natural.eps", t1, natural)
	quant_done()
	quant_init()
	f2=ExcelForestModel(dsn="forest", year=year, NO_FIRES=0, NO_LOGGING=1, NO_CULTURES=0)
	t2=f2.DoModel(Region, year)
	Bigglesplot("postscript","natural+cult.eps", t2,f2)
	quant_done()
	r=t2.Copy()
	f2.diff(natural, t2, t1, r)
	Bigglesplot("postscript","natural+cult-natural.eps", r,f2)
	# ND-Logging wo/Cultures
	quant_init()
	f3=ExcelForestModel(dsn="forest", year=year, NO_FIRES=0, NO_LOGGING=0, NO_CULTURES=1)
	t3=f3.DoModel(Region, year)
	quant_done()
	Bigglesplot("postscript","logging_wo_cult.eps", t3,f3)
	e1=t3.Copy()
	f3.diff(natural, t3, t1, e1)
	Bigglesplot("postscript","logging-natural_wo_cult.eps", e1,f3)
	# (ED-Logging+cultures)-(ED-Logging_wo_cultures)
	quant_init()
	f4=ExcelForestModel(dsn="forest", year=year, NO_FIRES=0, NO_LOGGING=0, NO_CULTURES=0)
	t4=f4.DoModel(Region, year)
	quant_done()
	e2=t4.Copy()
	f4.diff(natural, t4, t1, e2)
	Bigglesplot("postscript","natural-(logging+cult.eps)", e1,f3)
	e3=e2.Copy()
	f4.diff(f3, e2, e1, e3)
	Bigglesplot("postscript","with_cultures.ps", e1,f3)

main()



import types
#from math import *



class ModelParameter:
	def __init__(self, parameters_dict=None, primitive=0):
		if parameters_dict is None:
			pass
		else:
			self.__dict__.update(parameters_dict)
		self._primitive=primitive

	def __getitem__(self, index):
		return self.Convert(lambda x, p: x[p], index)

	def Convert(self, convert_func, parameter=None):
		answer=ModelParameter(primitive=self._primitive)
		for (key,value) in self.__dict__.items():
			if key[0]=="_" and key[1]!="_":
				continue
			#print "key:", key,
			if type(value)==types.InstanceType:
				setattr(answer, key, value.Convert(convert_func, parameter))
			else:
				setattr(answer, key, convert_func(value, parameter))
			#print
		return answer
	def ConvertWithNames(self, convert_func, name_convert, parameter=None):
		answer=ModelParameter(primitive=self._primitive)
		for (key,value) in self.__dict__.items():
			if key[0]=="_" and key[1]!="_":
				continue
			new_key=name_convert(value, key, parameter)
			if type(value)==types.InstanceType:
				setattr(answer, new_key, value.Convert(convert_func, parameter))
			else:
				setattr(answer, new_key, convert_func(value, parameter))
			#print
		return answer
	def Copy(self):
		return self.Convert(lambda x, p: x.__copy__())
	def __str__(self):
		return self.ReprTree("")

	def ReprTree(self, branchName):
		s=""
		items=self.__dict__.items()
		items.sort()
		for (key,value) in items:
			if key[0]=="_" and key[1]!="_":
				continue
			if branchName is None:
				name=key
			else:
				name=".".join([branchName,key])
			if type(value)==types.InstanceType:
				#print "name:", key, value
				s+=value.ReprTree(name)
			else:
				s+="%s: %s\n" % (name, str(value))
		return s

	def ForEachDo(self, leaf_func, parameter, branch_func=None, branch=None):
		"""
		branch_func must return 0 if not to go down the branches
		if branch_func==None then the search always go down to the leaves
		"""
		items=self.__dict__.items()
		items.sort()
		for (key,value) in items:
			if branch is None:
				br=[key]
			else:
				br=branch+[key]
			if type(value)==types.InstanceType:
				if branch_func is None or branch_func(br, value, parameter):
					value.ForEachDo(leaf_func, parameter, branch_func, br)
			else:
				leaf_func(br, value, parameter)
	def values(self):
		items=self.__dict__.items()
		list=[]
		for (key,value) in items:
			list.append(value)
		return list

	def __add__(self, other):
		return self.interpret(lambda x,y: x+y, [other])
	def __sub__(self, other):
		return self.interpret(lambda x,y: x-y, [other])
	def __mul__(self, other):
		return self.interpret(lambda x,y: x*y, [other])
	def __div__(self, other):
		return self.interpret(self._div_, [other])
	def _div_(self, x,y):
		try:
			return x/y
		except OverflowError:
			print "overflow :", y
			return 0
	def __abs__(self):
		return self.interpret(lambda x: abs(x), [])
	def __neg__(self):
		return self.interpret(lambda x: -x, [])
	def __pos__(self):
		return self.interpret(lambda x: x, [])
	def __int__(self):
		return self.interpret(lambda x: int(x), [])
	def __long__(self):
		return self.interpret(lambda x: long(x), [])
	def __float__(self):
		return self.interpret(lambda x: float(x), [])
	
	def dive(self, func, others):
		items=self.__dict__.items()
		items.sort()
		n=ModelParameter()
		for (key,value) in items:
			if key[0]=="_" and key[1]!="_":
				continue
			o=[]
			for other in others:
				if type(other)==types.InstanceType and other._primitive==0:
					o.append(getattr(other,key))
				else:
					o.append(other)

			if type(value)==types.InstanceType and not value._primitive:
				setattr(n, key, value.dive(func, o))
			else:
				params=[value]+o
				#print "params:", params
				dict={}
				for i in range(len(params)):
					dict["a%s" % str(i)]=params[i]

				setattr(n, key, ModelParameter({func: ModelParameter(dict)}, primitive=1))

		return n

	def interpret(self, func, others):
		items=self.__dict__.items()
		items.sort()
		n=ModelParameter()
		for (key,value) in items:
			if key[0]=="_" and key[1]!="_":
				continue
			o=[]
			_break=0
			for other in others:
				if type(other)==types.InstanceType and other._primitive==0:
					try:
						vv=getattr(other,key)
						o.append(vv)
					except AttributeError:
						_break=1
				else:
					o.append(other)
				if _break:
					break
			if _break:
				continue

			if type(value)==types.InstanceType and not value._primitive:
				setattr(n, key, value.interpret(func, o))
			else:
				params=[value]+o
				setattr(n, key, apply(func, params))

		return n
	def save(self, file, var_name=None):
		if type(file)==types.StringType:
			f=open(file, "w")
			need_closed=1
		else:
			f=file
			need_closed=0
		if var_name:
			f.write("%s=" % var_name)
		self._repr_data(f)
		if need_closed:
			f.close()
	def _repr_data(self, file):
		file.write(repr(self))
		file.write("\n")
	def __repr__(self):
		keys=self.__dict__.keys()
		keys.sort()
		s="%s({" % self.__class__.__name__
		for key in keys:
			val=self.__dict__[key]
			s+='%s:%s,' % (repr(key), repr(val))
		s+="})"
		return s
		

#def sin(x):
#	return x.interpret(lambda x: sin(x),[])


def test():
	a=ModelParameter({"S":1, "D":2})
	b=ModelParameter({"S":20})
	#s=repr(a)
	#print s, eval(s)
	f=open("t.dta","w")
	a.save(f, "name")
	a.save(f, "name2")
	f.close
	
if __name__=="__main__":
	test()
	
	


#!/bin/env python2
# encoding: utf-8
from dm import *
from numpy import *
import types
import math
import xlrd

TableKind={
	101:"СОСНА",
	102:"ЕЛЬ",
	103:"ПИХТА",
	104:"ЛИСТВЕННИЦА",
	105:"КЕДР",
	107:"ИТОГО ХВОЙНЫХ",
	124:"БЕРЕЗА",
	125:"ОСИНА",
	126:"ОЛЬХА СЕРАЯ",
	131:"ТОПОЛЬ",
	132:"ИВЫ ДРЕВОВИДНЫЕ",
	133:"ИТОГО МЯГКОЛИСТВЕННЫХ",
}

TableKindTranslit={
	101:"SOSNA",
	102:"EL",
	103:"PIKHTA",
	104:"LISTVENNITSA",
	105:"KEDR",
	107:"ITOGO KHVOINYH",
	124:"BEREZA",
	125:"OSINA",
	126:"OLKHA SERAYA",
	131:"TOPOL",
	132:"IVY DREVOVIDNYYE",
	133:"ITOGO MYAKGOLISTVENNYH",
}

TableEncoding='utf-8'

ForestKind={
	101:1,
	102:1,
	103:1,
	104:1,
	105:1,
	124:1,
	125:1,
	126:1,
	131:1,
	132:1,
}

VarNames={
	"Kind":1,
	"S1":1,
	"S2":1,
	"Ssr":1,
	"Spr":1,
	"Ssp":1,
	"Sper":1,
	"SIspper":1,
	"V1":1,
	"V2":1,
	"Vsr":1,
	"Vpr":1,
	"Vsp":1,
	"Vper":1,
	"VIspper":1,
	"L01":1,
	"Lk1":1,
	"L12":1,
	"L23":1,
	"L34":1,
	"L45":1,
	"L56":1,
	"Ssp101":1,
	"Ssp102":1,
	"Ssp103":1,
	"Ssp104":1,
	"Ssp105":1,
	"Ssp124":1,
	"Ssp125":1,
	"Sper101":1,
	"Sper102":1,
	"Sper103":1,
	"Sper104":1,
	"Sper105":1,
	"Sper124":1,
	"Sper125":1,
}

Transitions={
	101:20,
	102:20,
	103:20,
	104:20,
	105:40,
	124:10,
	125:10,
	126:10,
	131:10,
	132:10,
};

kind_exchange=100

logging=0.02

def f__add__(a,b):
	return a+b

FuncTransTable={
	"__add__": lambda x,y: x+y,
	"__sub__": lambda x,y: x-y,
	"__mul__": lambda x,y: x*y,
	"__div__": lambda x,y: x/y,
#	"__float__": lambda x: float(x),
}

def SpecialName(s):
	if s=="":
		return 1
	if s[0]=="_":
		if len(s)>1 and s[1]!="_":
			return 1
		else:
			return 0
		return 1
	return 0

def Interpret(func, value):
	global FuncTransTable
	dct={}
	dct.update(value.__dict__)
	del dct["_primitive"]
	args=dct.items()
	args.sort()
	args=map(lambda x: x[1], args)	# get values
	a=[]
	for arg in args:
		if type(arg)==types.InstanceType:
			for (key,val) in arg.__dict__.items():
				if key[:2]=="__":
					res=Interpret(key, val)
					# print "val:", val, "res:", res, "key:", key
					a.append(res)
					break	# should be only one
		else:
			a.append(arg)
	return apply(FuncTransTable[func], a)	# a function call

def NewItem():
#	return DMItem(DModel.NewItem())
	return DModel.NewItem()

class ModelParameter:
    def __init__(self, parameters_dict=None, primitive=0):
		if parameters_dict is None:
			pass
		else:
			self.__dict__.update(parameters_dict)
			self._keys=parameters_dict.keys()
		self._primitive=primitive

    def keys(self):
        return self._keys

    def item(self, key):
        return getattr(self, key)

    def Convert(self, convert_func, parameter=None):
		answer=ModelParameter(primitive=self._primitive)
		for (key,value) in self.__dict__.items():
			if key[0]=="_" and key[1]!="_":
				continue
			#print "key:", key,
			if type(value)==types.InstanceType:
				setattr(answer, key, value.Convert(convert_func, parameter))
			else:
				setattr(answer, key, convert_func(value, parameter))
			#print
		return answer

    def __repr__(self):
		return self.ReprTree("")

    def ReprTree(self, branchName):
		s=""
		items=self.__dict__.items()
		items.sort()
		for (key,value) in items:
			if key[0]=="_" and key[1]!="_":
				continue
			if branchName is None:
				name=key
			else:
				name=".".join([branchName,key])
			if type(value)==types.InstanceType:
				#print "name:", key, value
				s+=value.ReprTree(name)
			else:
				s+="%s: %s\n" % (name, str(value))
		return s

    def ForEachDo(self, leaf_func, parameter=None, branch_func=None, branch=None):
        """
        branch_func must return 0 if not to go down the branches
        if branch_func==None then the search always go down to the leaves
        """
        items=self.__dict__.items()
        items.sort()
        for (key,value) in items:
            if key.startswith("_"):
                continue
            if branch is None:
                br=[key]
            else:
                br=branch+[key]
            if type(value)==types.InstanceType:
                if branch_func is None or branch_func(br, value, parameter):
					value.ForEachDo(leaf_func, parameter, branch_func, br)
            else:
				leaf_func(br, value, parameter)

    def __add__(self, other):
		return self.interpret("__add__", [other])
    def __sub__(self, other):
		return self.interpret("__sub__", [other])
    def __mul__(self, other):
		return self.interpret("__mul__", [other])
    def __div__(self, other):
		return self.interpret("__div__", [other])
#	def __float__(self, other):
#		return self.interpret("__float__", [other])

    def dive(self, func, others):
		items=self.__dict__.items()
		items.sort()
		n=ModelParameter()
		for (key,value) in items:
			if key[0]=="_" and key[1]!="_":
				continue
			o=[]
			for other in others:
				if type(other)==types.InstanceType and other._primitive==0:
					o.append(getattr(other,key))
				else:
					o.append(other)

			if type(value)==types.InstanceType and not value._primitive:
				setattr(n, key, value.dive(func, o))
			else:
				params=[value]+o
				#print "params:", params
				dict={}
				for i in range(len(params)):
					dict["a%s" % str(i)]=params[i]

				setattr(n, key, ModelParameter({func: ModelParameter(dict)}, primitive=1))

		return n

    def interpret(self, func, others):
		items=self.__dict__.items()
		items.sort()
		n=ModelParameter()
		for (key,value) in items:
			if key[0]=="_" and key[1]!="_":
				continue
			o=[]
			for other in others:
				if type(other)==types.InstanceType and other._primitive==0:
					o.append(getattr(other,key))
				else:
					o.append(other)

			if type(value)==types.InstanceType and not value._primitive:
				setattr(n, key, value.interpret(func, o))
			else:
				params=[value]+o
				setattr(n, key, apply(FuncTransTable[func], params))

		return n


class TreeKind:
	def __init__(self, data_row, descrs):
		self.AcceptData(data_row, descrs)
		self.initialize()

	def AcceptData(self, data, descrs):
		global VarNames
		pos=0
		for descr in descrs:
			name=descr
			try:
				VarNames[name]
				self.__dict__[name]=data[pos]
			except KeyError:
				pass
			pos += 1

	def AcceptNaturalTransitionConstants(self, data, descrs):
		return self.AcceptData(data, descrs)

	def AcceptInterchangeConstants(self, data, descrs):
		return self.AcceptData(data, descrs)

	def initialize(self):
		self.normalize()
		# figuring out average concentration
		self.R1=self.V1/self.S1
		self.R2=self.V2/self.S2
		self.Rsr=self.Vsr/self.Ssr
		self.Rpr=self.Vpr/self.Spr
		self.Rsp=self.Vsp/self.Ssp
		self.Rper=self.Vper/self.Sper
		self.RIspper=self.VIspper/self.SIspper	# just for a case
		# building model graph

	def normalize(self):
		try:
			self.SIspper=self.Ssp+self.Sper
		except AttributeError:
			self.Ssp=self.SIspper-self.Sper
		try:
			self.VIspper=self.Vsp+self.Vper
		except AttributeError:
			self.Vsp=self.VIspper-self.Vper
		self.Kind=int(self.Kind)

	def __str__(self):
		return "Forest: "+str(self.Kind)+"\n dict: "+str(self.__dict__)+"\n"

	def __repr__(self):
		return self.__str__()

	def _build(self, value):
		node=NewItem()
		node.value=value
		node.initValue=value
		return node

	def _conn(self, n1, n2, value):
		edge=NewItem()
		edge.value=value
		edge.initValue=value
		edge.next=n1.another
		n1.another=edge
		edge.another=n2

	def BuildModel(self, no_forest, othernodes=None):
		global Transitions
		#trans=1.0/Transitions[self.Kind]
		self.trajectories=None
		self.model=ModelParameter({"S":ModelParameter(), "V":ModelParameter()})
		self.model.S.I=self._build(self.S1); self._conn(no_forest,self.model.S.I, self.L01); # zero state!!!
		self.model._first=self.model.S.I
		self.model.S.II=self._build(self.S2); self._conn(self.model.S.I,self.model.S.II, self.L12)
		self.model.S.I.next=self.model.S.II
		self.model.S.Sr=self._build(self.Ssr); self._conn(self.model.S.II,self.model.S.Sr, self.L23)
		self.model.S.II.next=self.model.S.Sr
		self.model.S.Pr=self._build(self.Spr); self._conn(self.model.S.Sr,self.model.S.Pr, self.L34)
		self.model.S.Sr.next=self.model.S.Pr
		self.model.S.Sp=self._build(self.Ssp); self._conn(self.model.S.Pr,self.model.S.Sp, self.L45)
		self.model.S.Pr.next=self.model.S.Sp
		self.model.S.Per=self._build(self.Sper); self._conn(self.model.S.Sp,self.model.S.Per, self.L56)
		self.model.S.Sp.next=self.model.S.Per

		self.model.S.Per.next=othernodes

		return self.model

	def PrepareStorage(self, obj):
		S=ModelParameter({
			"I":obj.__copy__(),
			"II":obj.__copy__(),
			"Sr":obj.__copy__(),
			"Pr":obj.__copy__(),
			"Sp":obj.__copy__(),
			"Per":obj.__copy__()
		})
		V=ModelParameter({
			"I":obj.__copy__(),
			"II":obj.__copy__(),
			"Sr":obj.__copy__(),
			"Pr":obj.__copy__(),
			"Sp":obj.__copy__(),
			"Per":obj.__copy__()
		})
		return ModelParameter({"S":S,"V":V})

	def FillInValues(self, number, struct):
		S=struct.S
		MS=self.model.S
		#print number, struct, sel.model
		S.I[number]=MS.I.value; S.II[number]=MS.II.value; S.Sr[number]=MS.Sr.value;
		S.Pr[number]=MS.Pr.value; S.Sp[number]=MS.Sp.value; S.Per[number]=MS.Per.value;
		V=struct.V
		MV=self.model.V
		V.I[number]=MV.I; 	V.II[number]=MV.II; V.Sr[number]=MV.Sr;
		V.Pr[number]=MV.Pr; V.Sp[number]=MV.Sp; V.Per[number]=MV.Per;

	def Reset(self):
		pass

	def AfterStates(self, modeller):
		assert 1
		self.model.V.I	= self.R1 * self.model.S.I.value
		self.model.V.II	= self.R2 * self.model.S.II.value
		self.model.V.Sr	= self.Rsr * self.model.S.Sr.value
		self.model.V.Pr	= self.Rpr * self.model.S.Pr.value
		self.model.V.Sp	= self.Rsp * self.model.S.Sp.value
		self.model.V.Per= self.Rper * self.model.S.Per.value


	def BeforeStates(self, modeller):
		pass


class ForestModel:
    def __init__(self, filename, year_id, Snel=0.0, Sempty=0.0, options=None):
        global ForestKind
        if options is None:
            self.options={}
        else:
            self.options=options

        xl=xlrd.open_workbook(filename, formatting_info=True)
        self.filename=filename

        s_page=xl.sheet_by_name("S%s" % year_id)
        g_page=xl.sheet_by_name("grow%s" % year_id)

        self.kinds=[]

        for rn in range(s_page.nrows):
            row=s_page.row_values(rn)
            if rn==0:
                s_struct=row
                continue
            try:
                ForestKind[int(row[0])]
                self.kinds.append(TreeKind(row, s_struct))
            except KeyError:
                pass

        self.normalize()

        for rn in range(g_page.nrows):
            row=g_page.row_values(rn)
            if rn==0:
                g_struct=row
                continue
            try:
                kind=self.kinds[int(row[0])] #hack!!!!
                kind.AcceptNaturalTransitionConstants(row, g_struct)
            except KeyError:
                pass

        del g_struct, s_struct
        del s_page, g_page

        f_page = xl.sheet_by_name("free%s" % year_id)
        struct = f_page.row_values(0)
        data = f_page.row_values(1)
        self.Sn=data[0]	# hack
        self.S0=data[1]	# hack
        self.n20=data[2]	# hack
        del f_page, data, struct

        c_page= xl.sheet_by_name("cultures")
        #cursor.execute('select year,dSk from "cultures$"')


        F101=self.kinds[101]
        Lk1=F101.Lk1
        S=0
        year_last=int(year_id)
        phase=1
        self.USk={}

        for rn in range(c_page.nrows):
            if rn==0: continue
            data = c_page.row_values(rn)

            y=int(data[0])
            dS=data[1]
            dS=dS/1000.0 # to make thousants of ga (kga)
            if y>year_last:
                phase=3
            if phase==1:
                y_p=y
                phase=2
                S=dS
            elif phase==2:
                y_t=y
                dy=y_t-y_p
                S = S - S * (Lk1**dy) + dS	#dSk
                y_p=y_t

            self.USk[y]=dS
            #print data[0], S, data[1]

        # print "Summary of Sk (kga):", S

        try:
            self.options["NO_CULTURES"]
            self.Sk0=0.0
            self.USk[year_last]=0.0
            print "Warning: No cultures!"
        except KeyError:
            self.Sk0=S
        del S,F101

        del data, c_page

        i_page = xl.sheet_by_name("interchange%s" % year_id)

        for rn in range(i_page.nrows):
            data=i_page.row_values(rn)
            if rn==0:
                struct=data
                continue
            try:
                kind=int(data[0])
                F=self.kinds[kind]
                F.AcceptInterchangeConstants(data,struct)
            except KeyError:
                pass
        del i_page, data

        l_page = xl.sheet_by_name('logging')
        self.ULogging={}
        for rn in range(l_page.nrows):
            data=l_page.row_values(rn)
            if rn==0: continue
            self.ULogging[int(data[0])]=data[1] / 1000.0	# conversion to kga
        del l_page, data

        # Cultural activity of habitants
        data=xl.sheet_by_name("people%s" % year_id).row_values(1)
        self.Sall=data[0]	# Al squaresl of the region (thousands of ga)
        self.Ssx=data[1]	# Agricultural squares (thousands of ga)
        self.N=data[2]		# No of habitants (thousants)
        self.L=data[3]		# Length of roads (km)
        self.KdN=data[4]	# Trend of habitants grow, i.e., N_{i+1} = KdN*N_{i}
        self.KNn=data[5]	# Activity of habitants to get therritory from forest
        self.KdNsx=data[6]  # How mush kgas to take by 1 thousants of habitants
        del data

		# fires
        data=xl.sheet_by_name("fires%s" % year_id).row_values(1)
        self.KULogging=data[0]	# How much to fire when logging (0.88)
        self.KfN=data[1]			# Fires due to habitants
        self.KfL=data[2]			# Fires due to transport
        self.KfSsx=data[3]		# Fires due to Agricultural interprises
        self.dUf=data[4]			# Fires due to DRY storm
        del data


    def BuildModel(self, species=None):

        self.model=ModelParameter()
        if species is None:
            species=self.kinds.keys()

        # Transition from nonforest territory (S.Nel) to territory without (w/o) forest (S.O)
        self.model.S=self.S=ModelParameter()
        self.model.S.O=self.S.O=DMItem(self.S0)
        self.model.S.Nel=self.S.Nel=DMItem(self.Sn, self.S.O, self.S.O, self.n20)
        head=self.S.Nel
        # Natural forest grow
        for s in species:
            mdl=self.kinds[s]
            path=mdl.BuildModel(self.S.O, head)
            setattr(self.model,"f"+str(s),mdl.model)
            head=path._first
            del path._first

		# Forest curtures
        self.F101=self.kinds[101]
        self.F101.Sk=self.Sk0
        self.F101.model.S.K=DMItem(self.F101.Sk, head, self.F101.model.S.I, self.F101.Lk1)
        head=self.F101.model.S.K

		# Kind exchange
        try:
            self.options["NO_INTERCHANGE"]
            print "Warning no species interchange!"
        except KeyError:
            for s1 in species:
                m1=getattr(self.model, "f"+str(s1))
                k1=self.kinds[s1]
                for s2 in species:
                    k2=self.kinds[s2]
                    m2=getattr(self.model, "f"+str(s2))
                    DMConnect(m1.S.Per, m2.S.Sr, getattr(k1, "Sper%i" % s2))
                    DMConnect(m1.S.Sp, m2.S.Sr, getattr(k1, "Ssp%i" % s2))

        return head	# of the nodelist

    def Reset(self):
		for kind in self.kinds.values():
			kind.Reset()

    def PrepareStorage(self, obj):
		m=ModelParameter({"S":ModelParameter({"O":obj.__copy__(),"Nel":obj.__copy__()})})
		for (name,value) in self.kinds.items():
			setattr(m, "f"+str(name), value.PrepareStorage(obj))
			if name==101:
				m.f101.S.K=obj.__copy__()
		return m

    def BeforeStates(self, modeller):
		for kind in self.kinds.values():
			kind.BeforeStates(modeller)

    def AfterStates(self, modeller):
		for kind in self.kinds.values():
			kind.AfterStates(modeller)
		if modeller.step_no>0:
			try:
				self.options["NO_CULTURES"]
			except KeyError:
				self.CalculateCultures(modeller.dt)
			try:
				self.options["NO_GET"]
			except KeyError:
				self.CalculateGetTerritory(modeller.dt)
			try:
				self.options["NO_LOGGING"]
			except KeyError:
				self.CalculateLogging(modeller.dt)
			try:
				self.options["NO_FIRES"]
			except KeyError:
				self.CalculateFires(modeller.dt)
			self.Ssx += self.GetDSsx(modeller.dt)
			self.N += self.GetDN(modeller.dt)

    def normalize(self):
        d={}
        for kind in self.kinds:
            d[kind.Kind]=kind

        self.kinds=d

    def CalculateCultures(self, dt):
        uk = self.USk[1973] * dt
        self.S.O.value -= uk
        self.F101.model.S.K.value += uk

    def CalculateLogging(self, dt):
		Sper=0
		Ssp=0
		for (key, val) in self.kinds.items():
			if key!=105:	# Kedr
				Sper += val.model.S.Per.value
				Ssp += val.model.S.Sp.value

		Srub=self.ULogging[1975] * dt
		Sall=Sper + Ssp
		for (key, val) in self.kinds.items():
			if key!=105:	# Kedr
				dSper=Srub * val.model.S.Per.value / Sall
				dSsp=Srub * val.model.S.Sp.value / Sall
				val.model.S.Per.value -= dSper
				val.model.S.Sp.value -= dSsp
				self.S.O.value += dSper+dSsp

    def CalculateFires(self, dt):
		S=0
		states=["I","II", "Sr", "Pr", "Sp", "Per"]

		for (key, val) in self.kinds.items():
			v = val.model.S
			for state in states:
				s=getattr(v, state).value
				S += s
			if key==101:	# Sosna cultres
				S += v.K.value

		Sfires= self.KULogging * self.ULogging[1975] +\
			self.KfN * self.N + self.KfL * self.L + self.KfSsx * self.Ssx + self.dUf

		Sfires *= dt

		for (key, val) in self.kinds.items():
			v = val.model.S
			for state in states:
				n=getattr(v, state)
				ds=Sfires * n.value / S
				n.value -= ds
				self.S.O.value += ds

			if key==101:	# Sosna cultres
				ds=Sfires * v.K.value / S
				v.K.value -= ds
				self.S.O.value += ds

    def FillInValues(self, number, struct):
		for (name,kind) in self.kinds.items():
			kind.FillInValues(number, getattr(struct,"f"+str(name)))
		struct.S.O[number]=self.S.O.value
		struct.S.Nel[number]=self.S.Nel.value

    def GetDSsx(self, dt):
		return self.GetDN(dt) * self.KdNsx

    def GetDN(self, dt):
		# print self.N, self.KdN, dt
		dN = self.N * self.KdN * dt
		return dN

    def CalculateGetTerritory(self, dt):
		dSN=self.KNn * self.GetDN(dt)
		dS=self.GetDSsx(dt)
		dS += dSN

		states=["I","II", "Sr", "Pr", "Sp", "Per"]

		S=self.S.O.value
		for (key, val) in self.kinds.items():
			v = val.model.S
			for state in states:
				s=getattr(v, state).value
				S += s
			if key==101:	# Sosna cultres
				S += v.K.value

		for (key, val) in self.kinds.items():
			v = val.model.S
			for state in states:
				n=getattr(v, state)
				ds=dS * n.value / S
				n.value -= ds
				self.S.Nel.value += ds

			if key==101:	# Sosna cultres
				ds=dS * v.K.value / S
				v.K.value -= ds
				self.S.Nel.value += ds

		ds=self.S.O.value * dS / S
		self.S.Nel.value += ds
		self.S.O.value -= ds


	#def __del__(self):
	#	del self.conn

class ForestModeller:
    def __init__(self,
                 forest_model,
                 starttime=0.0,
                 endtime=1.0,
                 interval=1.0,
                 subdivisions=10.0):
        self.forest_model=forest_model
        self.starttime=starttime
        self.endtime=endtime
        self.interval=interval
        self.subdivisions=subdivisions
        self.dt=self.interval/self.subdivisions
        #print "Values_to_store:", self.values_to_watch
        self.model_head=self.forest_model.BuildModel()
        self.Reset()

    def Reset(self):
		self.time_moment=self.starttime
		self.step_no=0
		self.forest_model.Reset()
		self.forest_model.AfterStates(self)
		self.prepareStorage()

    def prepareStorage(self):
        i=(self.endtime-self.starttime) / self.interval
        self.range=int(math.ceil(i))+1
        self.time=arange(self.range, dtype=float_)*self.interval + self.starttime
        self.trajectories=self.forest_model.PrepareStorage(zeros(self.range, dtype=float_))
        self.FillInValues(0)

    def FillInValues(self, number):
		self.forest_model.FillInValues(number, self.trajectories)

    def StepInterval(self):
		for n in range(self.subdivisions):
			self.forest_model.BeforeStates(self)
			self.model_head.step(self.dt, 1)
			self.forest_model.AfterStates(self)
		self.step_no+=1
		return self.step_no

    def Model(self):
		while (self.StepInterval()<self.range): self.FillInValues(self.step_no)
		self.trajectories.time=self.time

    def _makeLegends(self, gpm, legends):
		def _leaf_func(fields, value, parameter):
			(legends, time, gpm)=parameter
			name=fields[-1]
			if name!="time" and not SpecialName(name):
				title=".".join(fields)
				legends.append(gpm.Data(time, value, title=title))
		def _branch_func(fields, value, parameter):
			(legends, time, gpm)=parameter
			name=fields[-1]
			global Interpret
			if name[0:2]=="__":	# specail function
				result=Interpret(name, value)
				legends.append(gpm.Data(time, result, title=".".join(fields)))
				return 0
			else:
				return 1

		_legends=[]
		legends.ForEachDo(_leaf_func, (_legends, self.trajectories.time, gpm), branch_func=_branch_func)
		return _legends


    def Gnuplot(self, device, file=None, legends=None, initial_text=""):
		import Gnuplot
		gp=Gnuplot.Gnuplot(debug=0)
		gp("set terminal %s monochrome" % device)
		gp(initial_text)
		gp("set data style linespoints")
		if file is None:
			pass
		else:
			gp("set output '%s'" % file)
		if legends is None:
			legends=self.trajectories

		_legends=self._makeLegends(Gnuplot, legends)
		apply(gp.plot, _legends)
		gp.reset()
		del gp



if __name__=="__main__":
	# Quantum (begin)

	F=ForestModel("forest", "1973",
#		options={"NO_LOGGING":None, "NO_FIRES":None, "NO_INTERCHANGE":None}
#		options={"NO_INTERCHANGE":None}
#		options={"NO_INTERCHANGE":None,"NO_LOGGING":None, "NO_FIRES":None, "NO_GET":None}
		options={"NO_INTERCHANGE":None,"NO_FIRES":None, "NO_GET":None}
	)
	Modeller=ForestModeller(F,
		endtime=50., starttime=0., interval=5., subdivisions=100)
	control=Modeller.model_head.total
	Modeller.Model()

	# Quantum (end)

	plot=ModelParameter(
		{
			"Sfree":Modeller.trajectories.S.O,
			"Snel":Modeller.trajectories.S.Nel
		}
	)

	def All(kind):
		return kind.I+kind.II+kind.Sr+kind.Pr+kind.Sp+kind.Per

	plot_max=ModelParameter(
		{
			"Sos":All(Modeller.trajectories.f101.S),
			"El":All(Modeller.trajectories.f102.S),
			"Pikhta":All(Modeller.trajectories.f103.S),
			"Listvennica":All(Modeller.trajectories.f104.S),
			"Kedr":All(Modeller.trajectories.f105.S),
			"Bereza":All(Modeller.trajectories.f124.S),
			"Osina":All(Modeller.trajectories.f125.S),
			"Free":(Modeller.trajectories.S)
		}
	)

	log_y_init="set logscale y"

	Modeller.Gnuplot("postscript","graph.eps", plot)
	Modeller.Gnuplot("postscript","S.eps", Modeller.trajectories.S)
	Modeller.Gnuplot("postscript","F101_S.eps", Modeller.trajectories.f101.S, initial_text=log_y_init)
#	Modeller.Gnuplot("postscript","Max.eps", plot_max, initial_text=log_y_init)
	Modeller.Gnuplot("postscript","Max.eps", plot_max)
	Modeller.Gnuplot("postscript","F101_V.eps", Modeller.trajectories.f101.V, initial_text=log_y_init)
	print "Control of total square:", abs(control-Modeller.model_head.total), " total:", Modeller.model_head.total

	"""
	# no cultures
	FN=ForestModel("forest", "1973",  options={"NO_CULTURES":1})
	ModellerN=ForestModeller(FN,
		endtime=100., starttime=0., interval=10., subdivisions=100)
	ModellerN.Model()
	plot2=ModelParameter(
		{
			"Sfree":ModellerN.trajectories.S.O,
			"Snel":ModellerN.trajectories.S.Nel
		}
	)

	ModellerN.Gnuplot("postscript","graph2.eps", plot2)
	ModellerN.Gnuplot("postscript","graph21.eps", plot2-plot)
	"""


	DModel.Finalize(1)



from Numeric import *
from mutils import *
from biomass import *
from free import *
from spelper import *
from logging import *

def open1(name, mode):
	f=open(name, mode)
	f.write("from mutils import *\nfrom Numeric import *\n")
	return f

next_data={
	"low":(0,"middle"),
	"middle":(0,"high"),
	"high":(1,"low"),
}

def get_next_descr(prev):
	answer=[]
	p=list(prev)
	p.reverse()
	carry=1
	for d in p:
		if carry:
			(carry, next)=next_data[d]
		else:
			next=d
		answer.append(next)

	if carry:
		#print "No more scenarios"
		raise RuntimeError, "no more scenarios"

	answer.reverse()
	return tuple(answer)

def get_name(name, descr):
	return name+"_".join(list(descr))

def get_first_scen(name):
	descr=("low","low","low","low")
	return (get_name(name,descr),descr)

def get_next_scen(name,prev_descr):
	descr=get_next_descr(prev_descr)
	return (get_name(name,descr), descr)

LAST=50

def biomass (name):
	name="biomass_"+name
	val=globals()[name]
	print val.V[LAST],

def free(name):
	name="free_"+name
	val=globals()[name]
	print val.S[LAST],
	
def spel_per(name):
	name="spelper_"+name
	val=globals()[name]
	print val.S[LAST],


def logging(name): 
	name="logging_"+name
	val=globals()[name]
	#print val
	v=val.V
	vv=v.logged_101 + v.logged_102 + v.logged_103 + v.logged_104+v.logged_125 + v.logged_124
	s=0
	for vi in vv:
		s+=vi
	print s,

def do_etwas(name, descr):
	print name, # the variable part of the variable
	biomass (name)
	free(name)
	spel_per(name)
	logging(name)
	print

def main():
	(name, descr)=get_first_scen('')
	while 1:
		do_etwas(name, descr)
		try:
			(name, descr)=get_next_scen('', descr)
		except RuntimeError:
			return

if __name__=="__main__":
	import sys
	f=open("_result.txt","w")
	stdout=sys.stdout
	sys.stdout=f
	main()
	print
	print "Legend:"
	print "Senario_name V_biomass S_free S_spel_per Sum_logging"
	sys.stdout=stdout
	f.close()




from mutils import *
from biomass_good import *
from spelper_good import *
from free_good import *
from logging_good import *
from coex_good import *
from Numeric import *

def SpecialName(s):
	if s=="":
		return 1
	if s[0]=="_":
		if len(s)>1 and s[1]!="_":
			return 1
		else:
			return 0
		return 1
	return 0

biggles_point_types=[
	"none",
	"dot",
	"plus",
	"asterisk",
	"circle",
	"cross",
	"square",
	"triangle",
	"diamond",
	"star",
	"inverted triangle",
	"starburst",
	"fancy plus",
	"fancy cross",
	"fancy square",
	"fancy diamond",
]

biggles_line_types=[
	"solid",			"dotted",
	"dotdashed",		"shortdashed",
	"longdashed",	"dotdotdashed",
	"dotdotdotdashed"
]

colors=[
	0x000000,
	0x0000FF,
	0x00FF00,
	0x00FFFF,
	0xFF0000,
	0xFF00FF,
	0xFFFF00,
	0x000080,
	0x008000,
	0x008080,
	0x800000,
	0x800080,
	0x808000
]


def _makeBigglesLegends(gpm, legends, model):
	def _leaf_func(fields, value, (legends, time, gpm, model)):
		name=fields[-1]
		if name!="time" and not SpecialName(name):
			if value[0]<-1e37:
				return
			title=".".join(fields)

			#n=len(legends) / 2
			n=len(legends) % (len(biggles_line_types)*len(colors))
			color=colors[n % len(colors)]
			#print color, type(color)
			line_type=biggles_line_types[n / len(colors)]
			#l=gpm.Curve(time, value, type=line_type, color=color, linewidth=3)
			l=(abs(value[-1]), time, value, line_type, color, 3, title)
			#l.label=title
			legends.append(l)
	def _branch_func(fields, value, (legends, time, gpm, model)):
		name=fields[-1]
		global Interpret
		if name[0:2]=="__":	# specail function
			result=Interpret(name, value)
			l=gpm.Data(time, result) # colors...
			l.title=".".join(fields)
			legends.append(l)
			return 0
		else:
			return 1

	_legends=[]
	legends.ForEachDo(_leaf_func, (_legends, legends.time, gpm, model), branch_func=_branch_func)
	return _legends

def BigglesPlot(device, file, legends, model=None, initial_text="", title=""):
	import biggles
	p=biggles.FramedPlot()
	#p(initial_text)
	#p("set data style linespoints")
	p.title=title
	p.xlabel=r'$t$'
	p.ylabel='Value'
	if legends is None:
		legends=self.trajectories

	_legends=_makeBigglesLegends(biggles, legends, model)
	_legends.sort()
	_legends.reverse()
	new=[]
	for l in _legends:
		(skip, time, value, line_type, color, width, title)=l
		ll=biggles.Curve(time, value, type=line_type, color=color, linewidth=width)
		ll.label=title
		p.add(ll)
		new.append(ll)
	scale=0.2
	k=biggles.PlotKey( 1.1, 1, new )
	p.add(k)
	if file:
		p.write_eps(file)
	else:
		p.show()
	del p
	
def _makeLegends(self, gpm, legends):
	def _leaf_func(fields, value, parameter):
		(legends, time, gpm)=parameter
		name=fields[-1]
		if name!="time" and not SpecialName(name):
			title=".".join(fields)
			legends.append(gpm.Data(time, value, title=title))
	def _branch_func(fields, value, parameter):
		(legends, time, gpm)=parameter
		name=fields[-1]
		global Interpret
		if name[0:2]=="__":	# specail function
			result=Interpret(name, value)
			legends.append(gpm.Data(time, result, title=".".join(fields)))
			return 0
		else:
			return 1

	_legends=[]
	legends.ForEachDo(_leaf_func, (_legends, self.trajectories.time, gpm), branch_func=_branch_func)
	return _legends


def GnuPlot(self, device, file=None, legends=None, initial_text=""):
	import Gnuplot
	gp=Gnuplot.Gnuplot(debug=0)
	print device
	gp("set terminal %s monochrome" % device)
	gp(initial_text)
	gp("set data style linespoints")
	if file is None:
		pass
	else:
		gp("set output '%s'" % file)
	if legends is None:
		legends=self.trajectories

	_legends=self._makeLegends(Gnuplot, legends)
	apply(gp.plot, _legends)
	gp.reset()
	del gp

	

def make_scen (name):
	g=globals()
	d={}
	l=len(name)
	for n in g.keys():
		if n[:l]==name:
			d[n]=g[n]
	return ModelParameter(d)
	
def test():
	scen=make_scen("biomass")
	scen.time=arange(51)
	BigglesPlot("postscript","biomass.eps", scen)
	scen=make_scen("free")
	scen.time=arange(51)
	BigglesPlot("postscript","free.eps", scen)
	scen=make_scen("logging")
	scen.time=arange(51)
	BigglesPlot("postscript","logging.eps", scen)
	scen=make_scen("spelper")
	scen.time=arange(51)
	BigglesPlot("postscript","spelper.eps", scen)
	scen=make_scen("coex")
	scen.time=arange(51)
	print scen
	BigglesPlot("postscript","coex.eps", scen)
	
if __name__=="__main__":
	test()


import DModel
def DMConnect(item, to, value):
	edge=DModel.NewItem()
	edge.another=to
	edge.value=value
	edge.initValue=value

	edge.next=item.another
	item.another=edge

	return edge

def DMItem(value, next=None, connect_to=None, intens=0.0):
	node=DModel.NewItem()
	node.value=value
	node.initValue=value
	node.next=next
	if connect_to is None:
		return node
	DMConnect(node, connect_to, intens)
	return node



#import quant._quant as _quant
import _quant

debug=1

def set_debug(v):
	debug=v

def wrap(address, str, cls):
	if address==0:
		return None
	return cls(_quant.ConvertFromPointer(address, str))

def wrapc(str, cls):
	if str is None:
		return None
	return cls(str)

class ForeignStruct:
	def __init__(self, this, removable=0):
		#print this
		#if this[10]=="p":
		#	ferttr
		self.this=this
		self.removable=removable
	def __del__(self):
		if self.removable:
			self.release()
	def release(self):
		pass
	__setmethods__ = {
	}
	def as_parameter(self):
		return self.this
	def __setattr__(self,name,value):
		if (name == "this") or (name == "thisown"):
			self.__dict__[name] = value
			return
		method = self.__class__.__setmethods__.get(name,None)
		if method:
			return method(self.this,value)
		try:
			return getattr(self, "set_"+name)(value)
		except AttributeError:
			self.__dict__[name] = value
	__getmethods__ = {
	}
	def __getattr__(self,name):
		method = self.__class__.__getmethods__.get(name,None)
		if method:
			return method(self.this)
		try:
			return self.__dict__[name]
		except KeyError:
			if name[:4]=="get_":
				raise AttributeError, "not found"
			#try:
			return getattr(self, "get_"+name)()
			#except AttributeError:
			#	raise AttributeError,'Attribute "%s" of class %s instance not found' % (name, self.__class__.__name__)
	def __repr__(self):
		return "<%s instance at %s>" % (self.__class__.__name__, self.this,)

class Descriptor(ForeignStruct):
	def __init__(self, this, removable=0):
		ForeignStruct.__init__(self, this, removable)
	__setmethods__ = {
		"name" : _quant.ATPDescr_name_set,
		"type" : _quant.ATPDescr_type_set,
		"arity" : _quant.ATPDescr_arity_set,
		"constructive" : _quant.ATPDescr_constructive_set,
	}
	__getmethods__ = {
		"name" : _quant.ATPDescr_name_get,
		"type" : _quant.ATPDescr_type_get,
		"arity" : _quant.ATPDescr_arity_get,
		"constructive" : _quant.ATPDescr_constructive_get,
	}
	def release(self):
		rc=_quant.atp_descr_destroy(self.this)
		self.this=None
		return rc
	def __repr__(self):
		t=self.type
		if t==_quant.ATP_TYPE_VAR:
			return self.name
		elif t==_quant.ATP_TYPE_META_VAR:
			return "<%s>" % self.name
		elif t==_quant.ATP_TYPE_CONST:
			return "%s|%s|" % (self.name, self.value)
		elif t==_quant.ATP_TYPE_FUNC:
			return "%s/%i|%s|f|cons:%i" % (self.name, self.arity, self.value,self.constructive)
		elif t==_quant.ATP_TYPE_PRED:
			return "%s/%i|%s|p|cons:%i" % (self.name, self.arity, self.value,self.constructive)
		else:
			s="%s/%i(%s)=%s" % (self.name, self.arity, self.defs, self.exp)
			return s
	def get_value(self):
		return _quant.atp_descr_get_value(self.this)
	def set_value(self, value):
		return _quant.atp_descr_set_value(self.this, value)
	def get_defs(self):
		sub=self._get_sub()
		return wrapc(_quant.sub_defs_get(sub), DefList)
	def get_exp(self):
		sub=self._get_sub()
		exp=_quant.sub_exp_get(sub)
		return wrapc(exp, ExpNode)
	def _get_data(self):
		return _quant.ATPDescr_data_get(self.this)
	def _get_sub(self):
		data=self._get_data()
		return _quant.descr_sub_get(data)
	def store(self):
		return store_descriptor(self)
	def prove(self, trace=None):
		exp=self.exp
		lnk=exp.link()
		pcf=lnk.as_pcf()
		pcf.reduce()
		print "reduced:"
		pcf.print_as_tree()
		pcf.prepare_to_inference()
		return (pcf.prove(trace), pcf)


def create_descriptor(name, descr_type, arity, value=None):
	this=_quant.atp_descr_new(name, descr_type, arity)
	d=Descriptor(this, 1)
	if not (value is None):
		d.value=value
	return d

def install_constant(name, value=None):
	d=create_descriptor(name, _quant.ATP_TYPE_CONST, 0, value)
	d.store()
	return d

def install_function(name, argn, value=None):
	d=create_descriptor(name, _quant.ATP_TYPE_FUNC, argn, value)
	d.store()
	return d

def install_predicate(name, argn, value=None):
	d=create_descriptor(name, _quant.ATP_TYPE_PRED, argn, value)
	d.store()
	return d

def get_descriptor(name, arity=0):
	return wrapc(_quant.atp_descr_global_lookup(name,arity), Descriptor)

def store_descriptor(descr):
	old=get_descriptor(descr.name, descr.arity)
	if old:
		print "Warning: there are already registered descriptor with the same params '%s' (skipping new one)" % old
		return
		# raise ValueError, "there are already registered descriptor with the same params '%s'" % old
	_quant.atp_descr_global_store(descr.this)
	descr.removable=0

class GNode(ForeignStruct):
	def __init__(self, this, clss):
		ForeignStruct.__init__(self, this)
		self.clss=clss
	__setmethods__ = {
		"cnext" : _quant._GNode_next_set,
		"cprev" : _quant._GNode_prev_set,
		"cparent" : _quant._GNode_parent_set,
		"cchildren" : _quant._GNode_children_set,
	}
	__getmethods__ = {
	#	"cdata" : _quant._GNode_data_get,
		"cnext" : _quant._GNode_next_get,
		"cprev" : _quant._GNode_prev_get,
		"cparent" : _quant._GNode_parent_get,
		"cchildren" : _quant._GNode_children_get,
	}
	def get_nchildren(self):
		return _quant.g_node_n_children(self.this)
	def _get_data(self, string_SWIG_type):
		return _quant.ConvertFromPointer(_quant._GNode_data_get(self.this), string_SWIG_type)
	def get_children(self):
		return wrapc(self.cchildren, self.clss[2])
	def get_prev(self):
		return wrapc(self.cprev, self.clss[1])
	def get_next(self):
		return wrapc(self.cnext, self.clss[1])
	def get_parent(self):
		return wrapc(self.cparent, self.clss[0])

class ExpNode(GNode):
	def __init__(self, this):
		GNode.__init__(self, this, (ExpNode,ExpNode,ExpNode))
	def get_data(self):
		return wrapc(self._get_data("_p_ATPExpData"), ExpData)
	def __repr__(self):
		data=self.get_data()

		defs=data.get_defs()
		sgn=data.sign
		s=""
		if sgn==_quant.ATP_SIGN_PC_E:
			s+="E:"+repr(defs)
		elif sgn==_quant.ATP_SIGN_PC_A:
			s+="A:"+repr(defs)
		elif sgn==_quant.ATP_SIGN_PRED or sgn==_quant.ATP_SIGN_SUB:
			if data.defs:
				s+=s+repr(defs)
		if sgn==_quant.ATP_SIGN_PC_E  or sgn==_quant.ATP_SIGN_PC_A:
			s+=" ["
		curr=self.children
		nc=self.nchildren
		ss=chr(sgn)+" "
		ncurr=1
		while curr:
			s+=repr(curr)
			if ncurr!=nc:
				s+=ss
			curr=curr.next
			ncurr+=1

		if sgn==_quant.ATP_SIGN_PC_E  or sgn==_quant.ATP_SIGN_PC_A:
			s+=" ]"
		return s
	def link(self):
		return wrapc(_quant.atp_exp_node_link(self.this), ExpNode)
	def as_pcf(self):
		return wrapc(_quant.atp_exp_node_2_pcf(self.this), PCFNode)


class ExpData(ForeignStruct):
	__setmethods__ = {
		"csign" : _quant.ATPExpData_sign_set,
#		"data" : _quant.ATPExpData_data_set,
	}
	__getmethods__ = {
		"csign" : _quant.ATPExpData_sign_get,
#		"data" : _quant.ATPExpData_data_get,
	}
	def get_defs(self):
		return DefList(_quant.ATPExpData_data_get(self.this))
	def get_sign(self):
		return ord(self.csign)

class GArray(ForeignStruct):
	def __init__(self, this, elem_size):
		ForeignStruct.__init__(self, this)
		self._size_=elem_size
	__setmethods__ = {
		"data" : _quant._GArray_data_set,
		"len" : _quant._GArray_len_set,
	}
	__getmethods__ = {
		"data" : _quant._GArray_data_get,
		"len" : _quant._GArray_len_get,
	}
	def __len__(self):
		return self.len
	def __getitem__(self, index):
		return _quant.g_array_index_p(self.this, index, self._size_)
	def __repr__(self):
		s="%s([" % self.__class__.__name__
		for index in range(len(self)):
			s=s+repr(self[index])+", "
		return s+"])"

class DefList(GArray):
	def __init__(self, this):
		GArray.__init__(self, this, elem_size=_quant.cvar.ATPDef_size)
	def __getitem__(self, index):
		return wrap(GArray.__getitem__(self, index), "_p_ATPDef", Def)
	def __repr__(self):
		s=""
		l=len(self)
		for index in range(l):
			s+=repr(self[index])+"_"+str(index)
			if index+1<l:
				s+=", "
			else:
				s+=" "
		return s
	def reduced(self, skip_free_func=1):
		answer=[]
		for i in range(len(self)):
			item=self[i]
			if skip_free_func:
				if item.type==_quant.ATP_TYPE_FUNC:
					continue
			red=item.reduced(number=i)
			if red is None:
				pass
			else:
				answer.append(red)
		return answer


class RefList(GArray):
	def __init__(self, this):
		GArray.__init__(self, this, elem_size=_quant.cvar.ATPRef_size)
	def __getitem__(self, index):
		return wrap(GArray.__getitem__(self, index), "_p_ATPRef", Ref)
	def __repr__(self):
		s=""
		l=len(self)
		for index in range(l):
			s+=repr(self[index])
			if index+1<l:
				s+=", "
		return s
	def reduced(self):
		answer=[]
		for i in range(len(self)):
			item=self[i]
			red=item.reduced()
			if 0: #red is None:
				pass
			else:
				answer.append(red)
		return answer

class Def(ForeignStruct):
	__setmethods__ = {
		"type" : _quant.ATPDef_type_set,
		"info" : _quant.ATPDef_info_set,
		#"args" : _quant.ATPDef_args_set,
	}
	__getmethods__ = {
		"type" : _quant.ATPDef_type_get,
		"info" : lambda x: Info(_quant.ATPDef_info_get(x)),
		#"args" : lambda x: RefList(_quant.ATPDef_args_get(x)),
	}
	def get_args(self):
		#return RefList(_quant.ATPDef_args_get(self.this))
		args=_quant.ATPDef_args_get(self.this)
		if args is None:
			return None
		return RefList(args)
	def __repr__(self):
		t=self.type
		if t==_quant.ATP_TYPE_NULL:
			return repr(None)
		elif t==_quant.ATP_TYPE_VAR:
			return self.info.name
		elif t==_quant.ATP_TYPE_META_VAR:
			return "M_"+self.info.name
		elif t==_quant.ATP_TYPE_CONST:
			#return "%s |%s| " % (self.info.descr.name, repr(self.info.descr.value))
			return "%s" % (repr(self.info.descr.value))
		elif t==_quant.ATP_TYPE_FUNC:
			return "%s(%s)" % (self.info.descr.name, self.args)
		elif t==_quant.ATP_TYPE_PRED:
			return "%s(%s)" % (self.info.descr.name, self.args)
		elif t==_quant.ATP_TYPE_SUB:
			return "%s(%s)" % (self.info.descr.name, self.args)
		else:
			return "<%s instance at %s type: %i, info: %s, args: %s>" % (self.__class__.__name__, self.this, self.type, self.info, self.args)
	def reduced(self, number):
		t=self.type
		if t==_quant.ATP_TYPE_NULL:
			return None
		elif t==_quant.ATP_TYPE_VAR:
			return Var(self.info.name+"_"+str(number))
		elif t==_quant.ATP_TYPE_META_VAR:
			return Var("meta_"+self.info.name+"_"+str(number))
		elif t==_quant.ATP_TYPE_CONST:
			return self.info.descr.value
		elif t==_quant.ATP_TYPE_FUNC:
			rargs=self.args.reduced()
			val=self.info.descr.value
			if val:
				try:
					_a=1
					answer=apply(val, rargs)
					_a=0
				finally:
					if _a:
						print "Argument were:", rargs,"obtained from:", self
				return answer
			return Func(self.info.descr.name, rargs)
		elif t==_quant.ATP_TYPE_PRED:
			rargs=self.args.reduced()
			val=self.info.descr.value
			if val:
				try:
					_a=1
					answer=apply(val, rargs)
					_a=0
				finally:
					if _a:
						print "Argument were:", rargs,"obtained from:", self
				return answer
			return Pred(self.info.descr.name, rargs)
		elif t==_quant.ATP_TYPE_SUB:
			return None
		else:
			return None


class Info(ForeignStruct):
	__setmethods__ = {
		"name" : _quant.ATPInfo_name_set,
		#"descr" : _quant.ATPInfo_descr_set,
	}
	__getmethods__ = {
		"name" : _quant.ATPInfo_name_get,
		#"descr" : lambda x: Descriptor(_quant.ATPInfo_descr_get(x)),
	}
	def get_descr(self):
		descr=_quant.ATPInfo_descr_get(self.this)
		d=Descriptor(descr)
		return d
	def __repr__(self):
		#return "<%s instance at %s name: %s, descr: %s>" % (self.__class__.__name__, self.this, self.name, self. descr)
		return "<%s instance at %s name: %s>" % (self.__class__.__name__, self.this, self.name)


class Ref(ForeignStruct):
#	__setmethods__ = {
#		"owner" : _quant.ATPRef_owner_set,
#		"offset" : _quant.ATPRef_offset_set,
#	}
	__getmethods__ = {
		"owner" : lambda x: DefList(_quant.ATPRef_owner_get(x)),
		"cowner" : _quant.ATPRef_owner_get,
		"offset" : _quant.ATPRef_offset_get,
	}
	def unref(self):
		owner=self.cowner
		offset=self.offset
		addr=_quant.g_array_index_p(owner, offset, _quant.cvar.ATPDef_size)
		object=_quant.ConvertFromPointer(addr, "_p_ATPDef")
		return Def(object)
	def get_def(self):
		return self.unref()
	def __repr__(self):
		return repr(self.unref())+"_"+str(self.offset)
	def reduced(self):
		return self.unref().reduced(self.offset)

class PCFNode(GNode):
	def __init__(self, this, clss=None):
		if clss is None:
			clss=(PCFNode, PCFNode, PCFNode)
		GNode.__init__(self, this, clss)
	def _get_data(self, string_SWIG_type):
		addr=_quant._GNode_data_get(self.this)
		if addr==1:
			addr=0
		return _quant.ConvertFromPointer(addr, string_SWIG_type)
	def get_data(self):
		return wrapc(self._get_data("_p__GArray"), DefList)
	def get_defs(self):
		return self.get_data()
	def __repr__(self):
		return self._repr_()
	def _repr_data(self, data):
		return repr(data)
	def next_sgn(self, prev):
		if prev=="A":
			return "E"
		else:
			return "A"
	def tran_sgn(self, sgn):
		return sgn
	def _repr_(self, sgn="A"):
		defs=self.get_data()
		sgn1=self.tran_sgn(sgn)
		s="%s: %s " % (sgn1, self._repr_data(defs))
		nc=self.nchildren
		if nc > 1:
			s+="( "
			ss="; "
		else:
			ss=" "
		curr=self.children
		nsgn=self.next_sgn(sgn)
		cc=1
		while curr:
			s+=curr._repr_(nsgn)
			if cc<nc:
				s+=ss
			curr=curr.next
		if nc > 1:
			s+=") "
		return s
	def print_as_tree(self, sgn="A", level=0):
		tabs="    "*level
		defs=self.get_data()
		sgn1=self.tran_sgn(sgn)
		s=tabs+"%s: %s " % (sgn1, self._repr_data(defs))
		print s
		#print tabs, "|"

		curr=self.children
		nsgn=self.next_sgn(sgn)
		while curr:
			curr.print_as_tree(nsgn, level+1)
			curr=curr.next
	def reduce(self):
		return _quant.atp_pcf_reduce(self.this)
	def prepare_to_inference(self):
		answer=_quant.atp_pcf_prepare_to_inference(self.this)
		self.clss=(PCFNode, PCFNode, Base)
		self.prepared=1
		return answer
	def is_prepared(self):
		try:
			return self.prepared
		except AttributeError:
			return 0
	def prove(self, trace=None):
		if not self.is_prepared():
			self.prepare_to_inference()
		ex=1
		while ex:
			rc=_quant.atp_pcf_root_step(self.this)
			if trace:
				answer = apply(trace, (self,))
				if not answer:
					break
			ex=rc==_quant.ATP_STEP_SUCCESS
		return rc
	def get_reduced_defs(self):
		defs=self.get_defs()
		if defs is None:
			return []
		return defs.reduced()
	def get_reduced_bases(self):
		if not self.is_prepared():
			self.prepare_to_inference()
		answer=[]
		curr=self.children
		while curr:
			answer.append(curr.get_reduced_defs())
			curr=curr.next
		return answer

class PosList(GArray):
	def __init__(self, this):
		GArray.__init__(self, this, _quant.cvar.guint_size) # size of int
	def __repr__(self):
		s=""
		for i in range(len(self)):
			s+=str(self[i])+","
		return s
	def __getitem__(self, index):
		addr=GArray.__getitem__(self, index)
		return _quant.guint_at_address(addr)

class PCFData(ForeignStruct):
	"""
	__setmethods__ = {
		"defs" : _quantc.ATPPCFData_defs_set,
		"pos" : _quantc.ATPPCFData_pos_set,
		"lo_edge" : _quantc.ATPPCFData_lo_edge_set,
		"hi_edge" : _quantc.ATPPCFData_hi_edge_set,
		"attrs" : _quantc.ATPPCFData_attrs_set,
	}
	"""
	__getmethods__ = {
		"cdefs" : _quant.ATPPCFData_defs_get,
		"cpos" : _quant.ATPPCFData_pos_get,
		"lo_edge" : _quant.ATPPCFData_lo_edge_get,
		"hi_edge" : _quant.ATPPCFData_hi_edge_get,
		#"attrs" : _quantc.ATPPCFData_attrs_get,
	}
	def get_defs(self):
		return wrapc(self.cdefs, DefList)
	def get_pos(self):
		return wrapc(self.cpos, PosList)
	def __repr__(self):
		s=repr(self.get_defs())+"["+repr(self.get_pos())+"] lo: %i hi: %i" % (self.lo_edge, self.hi_edge)
		return s



class Base(PCFNode):
	def __init__(self, this):
		PCFNode.__init__(self, this, (PCFNode, Base, Quest))
	def get_data(self):
		return wrapc(self._get_data("_p_ATPPCFData"), PCFData)
	def _repr_data(self, data):
		if debug:
			return repr(data)
		else:
			return repr(data.get_defs())
	def get_defs(self):
		return self.get_data().get_defs()
	def tran_sgn(self, prev):
		return "B"

class Quest(PCFNode):
	def __init__(self, this):
		PCFNode.__init__(self, this, (Base, Quest, PCFNode))
	def _repr_data(self, data):
		if debug:
			return repr(data)
		else:
			return repr(data.get_defs())
	def get_data(self):
		return wrapc(self._get_data("_p_ATPPCFData"), PCFData)
	def get_defs(self):
		return self.get_data().get_defs()
	def tran_sgn(self, prev):
		return "Q"

def reduce_read_lines(strings):
	"""Get "\"-delimited strings together replacing "\" by " "
	"""
	result=[]
	prev=0
	for str in strings:
		next=0
		s=str.strip()
		if s=="":
			prev=0
			continue
		if s[0]=="#":
			prev=0
			continue
		if s[-1]=="\\":
			s=s[:-1]+" "
			s=" "+s
			next=1
		if prev:
			result[-1]+=s
		else:
			result.append(s)
		prev=next
	return result

class Var:
	def __init__(self, name):
		self.name=name
	def __repr__(self):
		return self.name
	def __str__(self):
		return self.__repr__()

class Pred(Var):
	def __init__(self, name, args):
		Var.__init__(self, name)
		self.args=args
	def __repr__(self):
		return "%s%s" % (self.name, repr(tuple(self.args)))
class Func (Pred):
	pass

# translation

def translate_string_prim(str):
	return _quant.translate_input(str)
def translate_lines(lines):
	sl=reduce_read_lines(lines)
	for str in sl:
		translate_string_prim(str)
def translate_string(str):
	ls=str.splitlines()
	translate_lines(ls)

def quant_init():
	_quant.quant_init()

def quant_done():
	_quant.quant_done()

def trace(pcf):
	print "trace"
	pcf.print_as_tree()
	return 1
def test():
	quant_init()
	c=create_descriptor("one", _quant.ATP_TYPE_CONST, 0, 1)
	c.store()
	c=create_descriptor("two", _quant.ATP_TYPE_CONST, 0, 2)
	c.store()
	c=create_descriptor("three", _quant.ATP_TYPE_CONST, 0, 3)
	c.store()
	c=create_descriptor("four", _quant.ATP_TYPE_CONST, 0, 4)
	c.store()
	translate_string("""set steps_count 10000;
set trace 15;
set tree_write_depth 60;
set debug_level 3;
""")
	#translate_string("fm a=p(one)&(a:x[p(x)>q(x)]) & (q(one)>n(one)) & (n(one)>write(p1(one)))")
	#translate_string("fm a=p(f(one))&(a:x[p(f(x))>p(g(x))] & a:x[p(f(x))>p(f(f(x)))])")
	#translate_string("fm a=p(one, two)&(a:x[p(one,x)>e:y,z[p(y,f(z))]])")
	#translate_string("fm a=p(one, two)&(a:x[p(one,x)>e:y,z[p(y,f(x))]])")
	#translate_string("fm a=p(one), p(two) & (a:x,y[p(x),p(y)>q(x,y)])")
	translate_string("fm a=p(one), p(two) & (a:x,y[p(x,y)>q(x,y)])")
	#translate_string("fm a=p(one), p(two), p(three), p(four) & (a:x,y[p(x),p(y)>e:z[q(x,y)]])")
	#translate_string("fm a=e:o,o1[p(one), p(two), after(one, two), after(two, three), l(o,one, f(o, one)), l(o1,two, f(o1,one)) & (a:x,y,x1,s0[p(x),l(s0,x,f(s0,y)),after(y,x1)>e:s[l(s,x,f(s,x1))]])]")

	#translate_string("fm a=e:x,y[p(f(x)),p(f(y)) & a:y[p(f(y))>q(f(y))]]")
	#translate_string("fm a=e:x,y[p(x),p(y) & a:y[p(y)>q(y)]]")
	#translate_string("fm a=e:x,[p(f(x)) & (p(f(x))>q(f(x)))]")
	#translate_string("fm a=e:x,y,z[a(y),a(z),p(x) & a:y,x[p(x),a(y) > c(x,y)]]")
	#translate_string("fm a=e:a,b[p(one,a),p(two,b)&(a:x[p(one,x)>q(x)])]")
	#translate_string("fm a=e:x,y,z,k[p(x,y), p(y,z) & a:a,b,c[p(a,b),p(b,c)>q(a,b,c)]]")
	#!!!!!!!!!translate_string("fm a=p & (p>q)")
	#translate_string("fm a=e:[p(one),p(two),p(three),p(four) & a:x[b(x,two)>q(x)]]")
	#descr_b=get_descriptor("b",2)
	#descr_b.constructive=1
	#print "descr_b:", descr_b
	descr=get_descriptor("a", 0)
	dp2=get_descriptor("p",2)
	dp2.constructive=1
	print dp2
	kb=descr.exp
	lnk=kb.link()
	pcf=lnk.as_pcf()
	pcf.reduce()
	pcf.print_as_tree()
	pcf.prove(trace)
	#pcf.print_as_tree()
	bases=pcf.get_reduced_bases()
	print bases


	raise SystemExit, "exit"
	f=open("fwin.fpc")
	lines=f.readlines()
	lines=reduce_read_lines(lines)
	for line in lines:
		print line
		_quant.translate_input(line)
	f.close()

	descr=get_descriptor("Cultures", 9)
	descr=get_descriptor("KB")
	square=get_descriptor("Square",2)
	square.value="12345"
	print
	print "square:",  square
	kb=descr.exp
	lnk=kb.link()
	pcf=lnk.as_pcf()
	pcf.reduce()
	#pcf.print_as_tree()
	pcf.prepare_to_inference()
	print "before --------------"
	pcf.print_as_tree()
	print "start proving --------------"
	pcf.prove()
	print "end proving --------------"
	pcf.print_as_tree()


if __name__=="__main__":
	test()



from ez_setup import use_setuptools
use_setuptools()
from setuptools import setup, find_packages, Extension

from distutils import log

log.set_verbosity(100)

setup( zip_safe = True, name="icc.dme.fd", version="0.0.5",
       author="Evgeny Cherkashin", author_email="eugeneai@irnok.net",
       description="Dynamic Modelling Environment",

       packages=find_packages("src"),
       package_dir={"": "src"},
       namespace_packages=["icc"],

       install_requires=[
           "setuptools"
       ],

       ext_modules=[
           Extension("icc.dme.fd.DModel",
                     sources=["src/icc/dme/fd/C/DModel.c"],
                 )
       ],

       license = "GNU GPL",
       keywords = "forest resources dynamics analysis tool application",

       long_description = """ """,

       # platform = "Os Independent.",
       # could also include long_description, download_url, classifiers, etc.
)



from Numeric import *
from mutils import *
from biomass import *
from free import *
from spelper import *

def open1(name, mode):
	f=open(name, mode)
	f.write("from mutils import *\nfrom Numeric import *\n")
	return f

next_data={
	"low":(0,"middle"),
	"middle":(0,"high"),
	"high":(1,"low"),
}

def get_next_descr(prev):
	answer=[]
	p=list(prev)
	p.reverse()
	carry=1
	for d in p:
		if carry:
			(carry, next)=next_data[d]
		else:
			next=d
		answer.append(next)

	if carry:
		print "No more scenarios"
		raise RuntimeError, "no more scenarios"

	answer.reverse()
	return tuple(answer)

def get_name(name, descr):
	return name+"_".join(list(descr))

def get_first_scen(name):
	descr=("low","low","low","low")
	return (get_name(name,descr),descr)

def get_next_scen(name,prev_descr):
	descr=get_next_descr(prev_descr)
	return (get_name(name,descr), descr)

def biomass ():
	pname="biomass_"
	(name, descr)=get_first_scen(pname)
	n=1
	f=open1("biomass_good.py", "w")
	while n:
		bio=globals()[name]
		prev=None
		skip=0
		for i in range(len(bio.V)):
			v=bio.V[i]
			if prev is None:
				prev=v
				continue
			if v<prev:
				skip=1
				break
			prev=v
		if not skip:
			print "saved", name
			bio.save(f,name)
		else:
			print "Skip", name
		try:
			(name, descr)=get_next_scen(pname, descr)
		except RuntimeError:
			n=0
	f.close()

def free():
	pname="free_"
	(name, descr)=get_first_scen(pname)
	n=1
	f=open1("free_good.py", "w")
	while n:
		sq=globals()[name]
		prev=None
		skip=0
		for i in range(len(sq.S)):
			v=sq.S[i]
			if prev is None:
				prev=v
				continue
			if v>prev:
				skip=1
				break
			prev=v
		if not skip:
			print "saved", name
			sq.save(f,name)
		else:
			print "Skip", name
		try:
			(name, descr)=get_next_scen(pname, descr)
		except RuntimeError:
			n=0
	f.close()

def spel_per():
	pname="spelper_"
	(name, descr)=get_first_scen(pname)
	n=1
	f=open1("spelper_good.py", "w")
	while n:
		sq=globals()[name]
		prev=None
		skip=0
		for i in range(len(sq.S)):
			v=sq.S[i]
			if prev is None:
				prev=v
				continue
			if v<prev:
				skip=1
				break
			prev=v
		if not skip:
			print "saved", name
			sq.save(f,name)
		else:
			print "Skip", name
		try:
			(name, descr)=get_next_scen(pname, descr)
		except RuntimeError:
			n=0
	f.close()

from logging import *

def logging(): #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	# неуменьшение объема рубок
	LS=62.0
	pname="logging_"
	(name, descr)=get_first_scen(pname)
	n=1
	f=open1("logging_good.py", "w")
	l=51
	while n:
		sq=globals()[name]
		prev=None
		skip=0
		v=sq.V
		vv=v.logged_101 + v.logged_102 + v.logged_103 + v.logged_104+v.logged_125 + v.logged_124
		vv[0]=vv[1]
		#print vv
		#v=LS-v
		#print v
		for i in range(51):
			v=vv[i]
			if prev is None:
				prev=v
				continue
			if v<prev:
				skip=1
				break
			prev=v
		if not skip:
			print "saved", name
			vv=ModelParameter({"V":vv})
			vv.save(f,name)
		else:
			print "Skip", name
		try:
			(name, descr)=get_next_scen(pname, descr)
		except RuntimeError:
			n=0

	f.close()

from coex import *
PART_MAX=1
def check(v, (amin, amax)):
	for a in v:
		a=abs(a)
		#print "->", amin, a, amax
		if a <= amax and a>=amin :
			continue
		return 0
	return 1

def checkall(v):
	for a in v.__dict__.keys():
		k=a
		if k[0]=="_":
			continue
		a=v.__dict__[a]
		#print "->", k
		range=prep(a[0])
		if not check(a, range):
			return 0
	return 1 
	
def prep(v):
	v=abs(v)
	return (v-v*PART_MAX, v+v*PART_MAX)
	
def coexistance():
	pname="coex_"
	(name, descr)=get_first_scen(pname)
	n=1
	f=open1("coex_good.py", "w")
	while n:
		sq=globals()[name].m_59
		del sq.S.logged_101,sq.S.logged_102,sq.S.logged_103,sq.S.logged_104,sq.S.logged_124,sq.S.logged_125
		del sq.S.cult_101
		prev=None
		skip=0
		
		v=sq.S
		skip= not checkall(v)
		if not skip:
			print "saved", name
			sq.save(f,name)
		else:
			print "Skip", name
		try:
			(name, descr)=get_next_scen(pname, descr)
		except RuntimeError:
			n=0
	f.close()


if __name__=="__main__":
	#biomass()
	#free()
	#spel_per()
	#logging()
	coexistance()




from icc.dme.fd.dm import *
import sys
import jsonpickle

class BB(object):
    """
    """

    def __init__(self, ):
        """
        """



class TestConnect(object):
    """Test Class
    """

    def __init__(self):
        """Init
        """
        A=DMItem(1)
        B=DMItem(2)
        C=DMConnect(A, B, 0.1)
        self.y={0:BB()}
        self.z=self.y.items()
        self.b=BB()
        #self.c=C

def main():
    rc=None
    a=TestConnect()
    rc=jsonpickle.encode(a)
    print (rc)
    b=jsonpickle.decode(rc)
    print b.b

    """
    rc=gnosis.xml.pickle.dumps(a)
    print rc
    b=gnosis.xml.pickle.loads(rc)
    print b.__class__
    """


main()

quit()
